Package: baymedr
Title: Computation of Bayes Factors for Common Biomedical Designs
Version: 0.2
Date: 2026-05-04
Authors@R: c(
    person(given = "Maximilian",
           family = "Linde",
           role = c("aut", "cre"),
           email = "maximilian.linde.92@gmail.com",
           comment = c(ORCID = "0000-0001-8421-090X")),
    person(given = "Don",
           family = "van Ravenzwaaij",
           role= "aut",
           email = "d.van.ravenzwaaij@rug.nl",
           comment = c(ORCID = "0000-0002-5030-4091")),
    person(given = "Jorge N.",
           family = "Tendeiro",
           role = "aut",
           email = "tendeiro@hiroshima-u.ac.jp",
           comment = c(ORCID = "0000-0003-1660-3642")),
    person(given = "Quentin F.",
           family = "Gronau",
           role = "ctb",
           email = "q.f.gronau@uva.nl",
           comment = c(ORCID = "0000-0001-5510-6943")))
Description: BAYesian inference for MEDical designs in R. Functions for the 
    computation of Bayes factors for common biomedical research designs. 
    Implemented are functions to test the equivalence (equiv_bf), 
    non-inferiority (infer_bf), and superiority (super_bf) of an experimental 
    group compared to a control group on a continuous outcome measure, as well 
    as functions for simulating survival data and calculating a Bayes factor 
    for Cox proportional hazards models. Bayes factors for these tests can be 
    computed based on raw data or summary statistics.
Depends: R (>= 3.4.0)
Imports: doParallel, foreach, methods, parallel, pso, rms, stats,
        stringr, survival
Suggests: knitr, rmarkdown, testthat
License: GPL-3
Encoding: UTF-8
Language: en-US
URL: https://github.com/maxlinde/baymedr
BugReports: https://github.com/maxlinde/baymedr/issues
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-04 14:27:14 UTC; maxlinde
Author: Maximilian Linde [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-8421-090X>),
  Don van Ravenzwaaij [aut] (ORCID:
    <https://orcid.org/0000-0002-5030-4091>),
  Jorge N. Tendeiro [aut] (ORCID:
    <https://orcid.org/0000-0003-1660-3642>),
  Quentin F. Gronau [ctb] (ORCID:
    <https://orcid.org/0000-0001-5510-6943>)
Maintainer: Maximilian Linde <maximilian.linde.92@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-11 15:00:02 UTC
Built: R 4.5.2; ; 2026-05-11 16:12:08 UTC; unix

Package: baymedr
Title: Computation of Bayes Factors for Common Biomedical Designs
Version: 0.2
Date: 2026-05-04
Authors@R: c(
    person(given = "Maximilian",
           family = "Linde",
           role = c("aut", "cre"),
           email = "maximilian.linde.92@gmail.com",
           comment = c(ORCID = "0000-0001-8421-090X")),
    person(given = "Don",
           family = "van Ravenzwaaij",
           role= "aut",
           email = "d.van.ravenzwaaij@rug.nl",
           comment = c(ORCID = "0000-0002-5030-4091")),
    person(given = "Jorge N.",
           family = "Tendeiro",
           role = "aut",
           email = "tendeiro@hiroshima-u.ac.jp",
           comment = c(ORCID = "0000-0003-1660-3642")),
    person(given = "Quentin F.",
           family = "Gronau",
           role = "ctb",
           email = "q.f.gronau@uva.nl",
           comment = c(ORCID = "0000-0001-5510-6943")))
Description: BAYesian inference for MEDical designs in R. Functions for the 
    computation of Bayes factors for common biomedical research designs. 
    Implemented are functions to test the equivalence (equiv_bf), 
    non-inferiority (infer_bf), and superiority (super_bf) of an experimental 
    group compared to a control group on a continuous outcome measure, as well 
    as functions for simulating survival data and calculating a Bayes factor 
    for Cox proportional hazards models. Bayes factors for these tests can be 
    computed based on raw data or summary statistics.
Depends: R (>= 3.4.0)
Imports: doParallel, foreach, methods, parallel, pso, rms, stats,
        stringr, survival
Suggests: knitr, rmarkdown, testthat
License: GPL-3
Encoding: UTF-8
Language: en-US
URL: https://github.com/maxlinde/baymedr
BugReports: https://github.com/maxlinde/baymedr/issues
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-04 14:27:14 UTC; maxlinde
Author: Maximilian Linde [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-8421-090X>),
  Don van Ravenzwaaij [aut] (ORCID:
    <https://orcid.org/0000-0002-5030-4091>),
  Jorge N. Tendeiro [aut] (ORCID:
    <https://orcid.org/0000-0003-1660-3642>),
  Quentin F. Gronau [ctb] (ORCID:
    <https://orcid.org/0000-0001-5510-6943>)
Maintainer: Maximilian Linde <maximilian.linde.92@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-11 15:00:02 UTC
Built: R 4.6.0; ; 2026-05-11 16:01:58 UTC; unix

# Generated by roxygen2: do not edit by hand

export(coxph_bf)
export(coxph_data_sim)
export(equiv_bf)
export(get_bf)
export(infer_bf)
export(super_bf)
import(doParallel)
import(foreach)
import(methods)
import(parallel)
import(pso)
import(rms)
import(stats)
import(stringr)
import(survival)

# baymedr: BAYesian inference for MEDical designs in R

`baymedr` is an R package with the goal of providing researchers with
easy-to-use tools for the computation of Bayes factors for common
biomedical research designs. Implemented are functions to test the
equivalence (`equiv_bf()`), non-inferiority (`infer_bf()`), and
superiority (`super_bf()`) of an experimental group (e.g., a new
medication) compared to a control group (e.g., a placebo or an already
existing medication) on a continuous dependent variable, as well as
functions for simulating survival data (`coxph_data_sim()`) and
calculating a Bayes factor for Cox proportional hazards models
(`coxph_bf()`). A special focus of `baymedr` lies on a user-friendly
interface, so that a wide variety or researchers (i.e., not only
statisticians) can utilize `baymedr` for their analyses.

## Installation and attaching

To install `baymedr` use:

``` r
install.packages("baymedr")
```

You can install the latest development version of `baymedr` from
[GitHub](https://github.com/), using the `devtools` package, with:

``` r
# install.packages("devtools")
devtools::install_github("maxlinde/baymedr")
```

Subsequently, you can load `baymedr`, so that it is ready to use:

``` r
library(baymedr)
```

#' @import stats
posterior_t <- function(delta,
                        t,
                        n1,
                        n2 = NULL,
                        ind_samples = FALSE,
                        prior_loc,
                        prior_scale,
                        prior_df,
                        rel_tol = .Machine$double.eps^0.25) {
  neff <- ifelse(ind_samples,
                 n1 * n2 / (n1 + n2),
                 n1)
  nu <- ifelse(ind_samples,
               n1 + n2 - 2,
               n1 - 1)
  mu_delta <- prior_loc
  gamma <- prior_scale
  kappa <- prior_df
  numerator <- suppressWarnings(
    dt(
      x = t,
      df = nu,
      ncp = sqrt(neff) * delta) * 1 / gamma * dt(
        (delta - mu_delta) / gamma,
        df = kappa))
  denominator <- integrate(integrand_t,
                           lower = -Inf,
                           upper = Inf,
                           t = t,
                           n = neff,
                           nu = nu,
                           mu_delta = mu_delta,
                           gamma = gamma,
                           kappa = kappa,
                           rel.tol = rel_tol)$value
  out <- numerator / denominator
  out[is.na(out)] <- 0
  return(out)
}

#' Bayes factor for equivalence designs
#'
#' \code{\link{equiv_bf}} computes a Bayes factor for equivalence designs with a
#' continuous dependent variable.
#'
#' The equivalence design has the following hypotheses: The null hypothesis
#' (i.e., H0) states that the population means of the experimental group (e.g.,
#' a new medication) and the control group (e.g., a placebo or an already
#' existing medication) are (practically) equivalent; the alternative hypothesis
#' (i.e., H1) states that the population means of the two groups are not
#' equivalent. The dependent variable must be continuous.
#'
#' Since the main goal of \code{\link{equiv_bf}} is to establish equivalence,
#' the resulting Bayes factor quantifies evidence in favor of the null
#' hypothesis (i.e., BF01). Evidence for the alternative hypothesis can easily
#' be calculated by taking the reciprocal of the original Bayes factor (i.e.,
#' BF10 = 1 / BF01). Quantification of evidence in favor of the null hypothesis
#' is logically sound and legitimate within the Bayesian framework (see e.g.,
#' van Ravenzwaaij et al., 2019).
#'
#' \code{\link{equiv_bf}} can be utilized to calculate a Bayes factor based on
#' raw data (i.e., if arguments \code{x} and \code{y} are defined) or summary
#' statistics (i.e., if arguments \code{n_x}, \code{n_y}, \code{mean_x}, and
#' \code{mean_y} are defined). In the latter case, either values for the
#' arguments \code{sd_x} and \code{sd_y} \strong{OR} \code{ci_margin} and
#' \code{ci_level} can be supplied. Arguments with 'x' as a name or suffix
#' correspond to the control group, whereas arguments with 'y' as a name or
#' suffix correspond to the experimental group.
#'
#' The equivalence interval can be specified with the argument \code{interval}.
#' However, it is not compulsory to specify an equivalence interval (see van
#' Ravenzwaaij et al., 2019). The default value of the argument \code{interval}
#' is 0, indicating a point null hypothesis. If an interval is preferred, the
#' argument \code{interval} can be set in two ways: A \emph{symmetric} interval
#' can be defined by either specifying a numeric vector of length one (e.g., 0.1,
#' which is converted to c(-0.1, 0.1)) or a numeric vector of length two (e.g.,
#' c(-0.1, 0.1)); an \emph{asymmetric} interval can be defined by specifying a
#' numeric vector of length two (e.g., c(-0.1, 0.2)). It can be specified
#' whether the equivalence interval (i.e., \code{interval}) is given in
#' standardized or unstandardized units with the \code{interval_std} argument,
#' where TRUE, corresponding to standardized units, is the default.
#'
#' For the calculation of the Bayes factor, a Cauchy prior density centered on 0
#' is chosen for the effect size under the alternative hypothesis. The standard
#' Cauchy distribution, with a location parameter of 0 and a scale parameter of
#' 1, resembles a standard Normal distribution, except that the Cauchy
#' distribution has less mass at the center but heavier tails (Liang et al.,
#' 2008; Rouder et al., 2009). The argument \code{prior_scale} specifies the
#' width of the Cauchy prior, which corresponds to half of the interquartile
#' range. Thus, by adjusting the Cauchy prior scale with \code{prior_scale},
#' different ranges of expected effect sizes can be emphasized. The default
#' prior scale is set to r = 1 / sqrt(2).
#'
#' \code{\link{equiv_bf}} creates an S4 object of class
#' \linkS4class{baymedrEquivalence}, which has multiple slots/entries (e.g.,
#' type of data, prior scale, Bayes factor, etc.; see Value). If it is desired
#' to store or extract solely the Bayes factor, the user can do this with
#' \code{\link{get_bf}}, by setting the S4 object as an argument (see Examples).
#'
#' @param interval A numeric vector of length one or two, specifying the
#'   boundaries of the equivalence interval. If a numeric vector of length one
#'   is specified, a symmetric equivalence interval will be used (e.g., a 0.1 is
#'   equivalent to c(-0.1, 0.1)). A numeric vector of length two provides the
#'   possibility to specify an asymmetric equivalence interval (e.g., c(-0.1,
#'   0.2)). The default is 0, indicating a point null hypothesis rather than an
#'   interval (see Details).
#' @param interval_std A logical vector of length one, specifying whether the
#'   equivalence interval (i.e., \code{interval}) is given in standardized
#'   (TRUE; the default) or unstandardized (FALSE) units.
#' @inheritParams super_bf
#'
#' @return An S4 object of class \linkS4class{baymedrEquivalence} is returned.
#'   Contained are a description of the model and the resulting Bayes factor:
#'   \itemize{ \item test: The type of analysis \item hypotheses: A statement of
#'   the hypotheses \itemize{ \item h0: The null hypothesis \item h1: The
#'   alternative hypothesis} \item interval: Specification of the equivalence
#'   interval in standardized and unstandardized units \itemize{ \item
#'   lower_std: The standardized lower boundary of the equivalence interval
#'   \item upper_std: The standardized upper boundary of the equivalence
#'   interval \item lower_unstd: The unstandardized lower boundary of the
#'   equivalence interval \item upper_unstd: The unstandardized upper boundary
#'   of the equivalence interval} \item data: A description of the data
#'   \itemize{ \item type: The type of data ('raw' when arguments \code{x} and
#'   \code{y} are used or 'summary' when arguments \code{n_x}, \code{n_y},
#'   \code{mean_x}, \code{mean_y}, \code{sd_x}, and \code{sd_y} (or
#'   \code{ci_margin} and \code{ci_level} instead of \code{sd_x} and
#'   \code{sd_y}) are used) \item ...: values for the arguments used, depending
#'   on 'raw' or summary'} \item prior_scale: The width of the Cauchy prior
#'   distribution \item bf: The resulting Bayes factor } A summary of the model
#'   is shown by printing the object.
#'
#' @export
#' @import stats stringr
#'
#' @references
#'   Gronau, Q. F., Ly, A., & Wagenmakers, E.-J. (2020). Informed Bayesian
#'   t-tests. \emph{The American Statistician}, \emph{74}, 137-143.
#'
#'   Liang, F., Paulo, R., Molina, G., Clyde, M. A., & Berger, J. O. (2008).
#'   Mixtures of g priors for Bayesian variable selection. \emph{Journal of the
#'   American Statistical Association}, \emph{103}, 410-423.
#'
#'   Rouder, J. N., Speckman, P. L., Sun, D., Morey, R. D., & Iverson, G.
#'   (2009). Bayesian t tests for accepting and rejecting the null hypothesis.
#'   \emph{Psychonomic Bulletin & Review}, \emph{16}, 225-237.
#'
#'   van Ravenzwaaij, D., Monden, R., Tendeiro, J. N., & Ioannidis, J. P. A.
#'   (2019). Bayes factors for superiority, non-inferiority, and equivalence
#'   designs. \emph{BMC Medical Research Methodology}, \emph{19}, 71.
#'
#' @examples
#' ## equiv_bf using raw data:
#'
#' # Assign model to variable.
#' equiv_raw <- equiv_bf(x = rnorm(100, 10, 15),
#'                       y = rnorm(130, 13, 10))
#'
#' # Extract Bayes factor from variable.
#' get_bf(equiv_raw)
#'
#' # ----------
#' # ----------
#'
#' ## equiv_bf using summary statistics with data from Steiner et al. (2015).
#' ## With a point null hypothesis:
#'
#' # Assign model to variable.
#' equiv_sum_point <- equiv_bf(n_x = 560,
#'                             n_y = 538,
#'                             mean_x = 8.683,
#'                             mean_y = 8.516,
#'                             sd_x = 3.6,
#'                             sd_y = 3.6)
#'
#' # Extract Bayes factor from model.
#' get_bf(equiv_sum_point)
#'
#' # ----------
#' # ----------
#'
#' ## equiv_bf using summary statistics with data from Steiner et al. (2015).
#' ## With an interval null hypothesis:
#'
#' # Assign model to variable.
#' equiv_sum_interval <- equiv_bf(n_x = 560,
#'                                n_y = 538,
#'                                mean_x = 8.683,
#'                                mean_y = 8.516,
#'                                sd_x = 3.6,
#'                                sd_y = 3.6,
#'                                interval = 0.05)
#'
#' # Extract Bayes factor from model.
#' get_bf(equiv_sum_interval)
equiv_bf <- function(x = NULL,
                     y = NULL,
                     n_x = NULL,
                     n_y = NULL,
                     mean_x = NULL,
                     mean_y = NULL,
                     sd_x = NULL,
                     sd_y = NULL,
                     ci_margin = NULL,
                     ci_level = NULL,
                     interval = 0,
                     interval_std = TRUE,
                     prior_scale = 1 / sqrt(2)) {
  if (any(!is.null(x),
          !is.null(y)) && any(!is.null(n_x),
                              !is.null(n_y),
                              !is.null(mean_x),
                              !is.null(mean_y),
                              !is.null(sd_x),
                              !is.null(sd_y),
                              !is.null(ci_margin),
                              !is.null(ci_level))) {
    stop(str_c(
      "Only 'x' and 'y' OR 'n_x', 'n_y', 'mean_x', 'mean_y', 'sd_x', and ",
      "'sd_y' (or 'ci_margin' and 'ci_level' instead of 'sd_x' and 'sd_y') ",
      "must be defined."
    ),
    call. = FALSE)
  }
  if (xor(!is.null(x),
          !is.null(y))) {
    stop("Both 'x' and 'y' must be defined.",
         call. = FALSE)
  }
  if (any(!is.null(n_x),
          !is.null(n_y),
          !is.null(mean_x),
          !is.null(mean_y))) {
    if (any(is.null(n_x),
            is.null(n_y),
            is.null(mean_x),
            is.null(mean_y)) ||
        ((is.null(sd_x) || is.null(sd_y)) &&
         (is.null(ci_margin) || is.null(ci_level)))) {
      stop(str_c(
        "All 'n_x', 'n_y', 'mean_x', 'mean_y', 'sd_x', and 'sd_y' (or ",
        "'ci_margin' and 'ci_level' instead of 'sd_x' and 'sd_y') must be ",
        "defined."
      ),
      call. = FALSE)
    }

    if (!xor(!is.null(sd_x) && !is.null(sd_y),
             !is.null(ci_margin) && !is.null(ci_level))) {
      stop(str_c("Only 'sd_x' and 'sd_y' OR 'ci_margin' and 'ci_level' must ",
                  "be defined."),
           call. = FALSE)
    }
  }
  if (all(!is.null(n_x),
          !is.null(n_y),
          !is.null(mean_x),
          !is.null(mean_y)) && (xor(
            !is.null(sd_x) && !is.null(sd_y),
            !is.null(ci_margin) && !is.null(ci_level)
          ))) {
    if (!is.null(ci_level) && (length(ci_level) > 1 || ci_level <= 0 ||
                               ci_level >= 1 || !is.numeric(ci_level))) {
      stop("'ci_level' must be a single numeric value between 0 and 1.",
           call. = FALSE)
    }
    data <- list(type = "summary data",
                 data = list(n_x = n_x,
                             n_y = n_y,
                             mean_x = mean_x,
                             mean_y = mean_y,
                             sd_x = sd_x,
                             sd_y = sd_y,
                             ci_margin = ci_margin,
                             ci_level = ci_level))
  }
  if (!is.null(x) && !is.null(y)) {
    if (any(is.na(x)) || any(is.na(y))) {
      stop("'x' and 'y' must not contain missing values.",
           call. = FALSE)
    }
    if (any(is.infinite(x)) || any(is.infinite(y))) {
      stop("'x' and 'y' must not contain infinite values.",
           call. = FALSE)
    }
    if (!is.numeric(x) || !is.numeric(y)) {
      stop("'x' and 'y' must be numeric vectors.",
           call. = FALSE)
    }
    n_x <- length(x)
    n_y <- length(y)
    mean_x <- mean(x)
    mean_y <- mean(y)
    sd_x <- sd(x)
    sd_y <- sd(y)
    data <- list(type = "raw data",
                 data = list(x = x,
                             y = y))
  }
  if (!is.numeric(prior_scale) || length(prior_scale) > 1) {
    stop("'prior_scale' must be a single numeric value.",
         call. = FALSE)
  }
  if (!is.numeric(interval) || length(interval) > 2) {
    stop("'interval' must be a numeric vector of length one or two.",
         call. = FALSE)
  }
  if (!is.logical(interval_std) || length(interval_std) > 1) {
    stop("'interval_std' must be a single logical value.",
         call. = FALSE)
  }
  if (!is.null(sd_x) && !is.null(sd_y)) {
    sd_pooled <- sqrt(((n_x - 1) * sd_x ^ 2 + (n_y - 1) * sd_y ^ 2) /
                        (n_x + n_y - 2))
    se <- sd_pooled * sqrt(1 / n_x + 1 / n_y)
  } else {
    perc <- 1 - ((1 - ci_level) / 2)
    se <- ci_margin / qt(p = perc,
                         df = n_x + n_y - 2)
    sd_pooled <- se / sqrt(1 / n_x + 1 / n_y)
  }
  t_stat <- (mean_y - mean_x) / se
  if (isFALSE(interval_std)) {
    inter_std <- interval / sd_pooled
    inter_unstd <- interval
  } else {
    inter_std <- interval
    inter_unstd <- interval * sd_pooled
  }
  if (length(inter_std) == 1) {
    inter_std <- c(-inter_std, inter_std)
    inter_unstd <- c(-inter_unstd, inter_unstd)
  }
  if (identical(inter_std, c(0, 0))) {
    res <- bf10_t(t = t_stat,
                  n1 = n_x,
                  n2 = n_y,
                  ind_samples = TRUE,
                  prior_loc = 0,
                  prior_scale = prior_scale,
                  prior_df = 1)
    bf <- 1 / res$bf_10
    h0 <- "mu_y == mu_x"
    h1 <- "mu_y != mu_x"
  } else {
    cdf_t_upper <- cdf_t(x = inter_std[[2]],
                         t = t_stat,
                         n1 = n_x,
                         n2 = n_y,
                         ind_samples = TRUE,
                         prior_loc = 0,
                         prior_scale = prior_scale,
                         prior_df = 1)
    cdf_t_lower <- cdf_t(x = inter_std[[1]],
                         t = t_stat,
                         n1 = n_x,
                         n2 = n_y,
                         ind_samples = TRUE,
                         prior_loc = 0,
                         prior_scale = prior_scale,
                         prior_df = 1)
    post_dens <- cdf_t_upper - cdf_t_lower
    if (post_dens < 0) {
      post_dens <- 0
      warning(str_c(
        "Numerical integration yields a posterior density slightly lower ",
        "than 0. The posterior density has been replaced by 0."
      ),
      call. = FALSE)
    }
    prior_dens <- pcauchy(q = inter_std[[2]],
                          scale = prior_scale) - pcauchy(q = inter_std[[1]],
                                                         scale = prior_scale)
    bf <- (post_dens / prior_dens) /
      ((1 - post_dens) / (1 - prior_dens))
    h0 <- "mu_y - mu_x > c_low AND mu_y - mu_x < c_high"
    h1 <- "mu_y - mu_x < c_low OR mu_y - mu_x > c_high"
  }
  test <- "Equivalence analysis"
  hypotheses <- list(h0 = h0,
                     h1 = h1)
  interval <- list(lower_std = inter_std[[1]],
                   upper_std = inter_std[[2]],
                   lower_unstd = inter_unstd[[1]],
                   upper_unstd = inter_unstd[[2]])
  new(Class = "baymedrEquivalence",
      test = test,
      hypotheses = hypotheses,
      interval = interval,
      data = data,
      prior_scale = prior_scale,
      bf = bf)
}

context("test-equiv_bf")

con <- rnorm(100)
exp <- rnorm(100)

test_that("equiv_bf yields correct S4 class", {
  expect_is(
    equiv_bf(x = con,
             y = exp),
    "baymedrEquivalence"
  )
  expect_is(
    equiv_bf(n_x = 100,
             n_y = 100,
             mean_x = 10,
             mean_y = 10,
             sd_x = 5,
             sd_y = 5),
    "baymedrEquivalence"
  )
  expect_is(
    equiv_bf(n_x = 100,
             n_y = 100,
             mean_x = 10,
             mean_y = 10,
             ci_margin = 5,
             ci_level = 0.95),
    "baymedrEquivalence"
  )
})

test_that("equiv_bf yields numeric Bayes factor", {
  expect_true(
    is.numeric(equiv_bf(x = con,
                        y = exp)@bf)
  )
  expect_true(
    is.numeric(equiv_bf(n_x = 100,
                        n_y = 100,
                        mean_x = 10,
                        mean_y = 10,
                        sd_x = 5,
                        sd_y = 5)@bf)
  )
  expect_true(
    is.numeric(equiv_bf(n_x = 100,
                        n_y = 100,
                        mean_x = 10,
                        mean_y = 10,
                        ci_margin = 5,
                        ci_level = 0.95)@bf)
  )
})

test_that("equiv_bf gives correct error messages", {
  expect_error(
    equiv_bf(x = con,
             y = exp,
             ci_margin = 5),
    str_c(
      "Only 'x' and 'y' OR 'n_x', 'n_y', 'mean_x', 'mean_y', 'sd_x', and ",
      "'sd_y' (or 'ci_margin' and 'ci_level' instead of 'sd_x' and 'sd_y') ",
      "must be defined."
    ),
    fixed = TRUE
  )
  expect_error(
    equiv_bf(x = con),
    "Both 'x' and 'y' must be defined.",
    fixed = TRUE
  )
  expect_error(
    equiv_bf(n_x = 100,
             n_y = 100,
             mean_x = 10,
             sd_x = 5),
    str_c(
      "All 'n_x', 'n_y', 'mean_x', 'mean_y', 'sd_x', and 'sd_y' (or ",
      "'ci_margin' and 'ci_level' instead of 'sd_x' and 'sd_y') must be ",
      "defined."
    ),
    fixed = TRUE
  )
  expect_error(
    equiv_bf(n_x = 100,
             n_y = 100,
             mean_x = 10,
             mean_y = 10,
             sd_x = 5,
             sd_y = 5,
             ci_margin = 5,
             ci_level = 0.95),
    "Only 'sd_x' and 'sd_y' OR 'ci_margin' and 'ci_level' must be defined.",
    fixed = TRUE
  )
  expect_error(
    equiv_bf(x = con,
             y = c(exp[1:99], NA)),
    "'x' and 'y' must not contain missing values.",
    fixed = TRUE
  )
  expect_error(
    equiv_bf(x = con,
             y = c(exp[1:99], Inf)),
    "'x' and 'y' must not contain infinite values.",
    fixed = TRUE
  )
  expect_error(
    equiv_bf(x = con,
             y = rep(letters[1:10],
                     each = 10)),
    "'x' and 'y' must be numeric vectors.",
    fixed = TRUE
  )
  expect_error(
    equiv_bf(x = con,
             y = exp,
             prior_scale = "x"),
    "'prior_scale' must be a single numeric value.",
    fixed = TRUE
  )
  expect_error(
    equiv_bf(x = con,
             y = exp,
             prior_scale = c(1, 2)),
    "'prior_scale' must be a single numeric value.",
    fixed = TRUE
  )
  expect_error(
    equiv_bf(x = con,
             y = exp,
             interval = "2"),
    "'interval' must be a numeric vector of length one or two.",
    fixed = TRUE
  )
  expect_error(
    equiv_bf(x = con,
             y = exp,
             interval = c(1, 2, 3)),
    "'interval' must be a numeric vector of length one or two.",
    fixed = TRUE
  )
  expect_error(
    equiv_bf(n_x = 100,
             n_y = 100,
             mean_x = 0,
             mean_y = 2,
             ci_margin = 5,
             ci_level = 1.5),
    "'ci_level' must be a single numeric value between 0 and 1.",
    fixed = TRUE
  )
  expect_error(
    equiv_bf(n_x = 100,
             n_y = 100,
             mean_x = 0,
             mean_y = 2,
             ci_margin = 5,
             ci_level = "high"),
    "'ci_level' must be a single numeric value between 0 and 1.",
    fixed = TRUE
  )
  expect_error(
    equiv_bf(n_x = 100,
             n_y = 100,
             mean_x = 0,
             mean_y = 2,
             ci_margin = 5,
             ci_level = c(0.3, 0.7)),
    "'ci_level' must be a single numeric value between 0 and 1.",
    fixed = TRUE
  )
})

context("test-get_bf")

con <- rnorm(100)
exp <- rnorm(100)

mod_super <- super_bf(x = con,
                      y = exp)

mod_equiv <- equiv_bf(x = con,
                      y = exp)

mod_infer <- infer_bf(x = con,
                      y = exp,
                      ni_margin = 0.5)

data <- survival::aml
names(data) <- c("time", "event", "group")
data$group <- ifelse(test = data$group == "Maintained",
                     yes = 0,
                     no = 1)

mod_coxph <- coxph_bf(data = data)

sim_data <- coxph_data_sim(n_data = 3,
                           ns_c = 20,
                           ns_e = 56,
                           ne_c = 18,
                           ne_e = 40,
                           cox_hr = c(0.433, 0.242, 0.774),
                           cox_hr_ci_level = 0.95,
                           maxit = 25)

mod_coxph_multi <- coxph_bf(data = sim_data)

test_that("get_bf extracts numeric Bayes factor from S4 object", {
  expect_true(
    is.numeric(get_bf(mod_super))
  )
  expect_true(
    is.numeric(get_bf(mod_equiv))
  )
  expect_true(
    is.numeric(get_bf(mod_infer))
  )
  expect_true(
    is.numeric(get_bf(mod_coxph))
  )
  expect_true(
    is.numeric(get_bf(mod_coxph_multi))
  )
})

test_that("get_bf gives correct error messages", {
  expect_error(
    get_bf(t.test(x = con,
                  y = exp)),
    str_c(
      "Bayes factors can only be extracted from S4 objects of classes ",
      "'baymedrEquivalence', 'baymedrNonInferiority', ",
      "'baymedrSuperiority', 'baymedrCoxProportionalHazards', and ",
      "'baymedrCoxProportionalHazardsMulti'."
    ),
    fixed = TRUE
  )
})

#' @import methods
setMethod(
  f = "show",
  signature = "baymedrSuperiority",
  definition = function(object) {
    cat("******************************",
        "\n",
        object@test,
        "\n",
        rep("-",
            times = nchar(object@test)),
        "\n",
        "Data:                         ",
        object@data$type,
        "\n",
        "H0 (non-superiority):         ",
        object@hypotheses$h0,
        "\n",
        if (object@hypotheses$h1 == "mu_y < mu_x") {
          "H- (superiority):             "
        },
        if (object@hypotheses$h1 == "mu_y > mu_x") {
          "H+ (superiority):             "
        },
        object@hypotheses$h1,
        "\n",
        "Cauchy prior scale:           ",
        formatC(x = object@prior_scale,
                digits = 3,
                format = "f"),
        "\n\n",
        if (object@hypotheses$h1 == "mu_y < mu_x") {
          "    BF-0 (superiority) = "
        },
        if (object@hypotheses$h1 == "mu_y > mu_x") {
          "    BF+0 (superiority) = "
        },
        if (object@bf > 1 / 1000 && object@bf < 1000) {
          formatC(x = object@bf,
                  digits = 3,
                  format = "f")
        } else {
          formatC(x = object@bf,
                  digits = 3,
                  format = "e")
        },
        "\n",
        "******************************",
        "\n",
        sep = "")
  }
)

setMethod(
  f = "show",
  signature = "baymedrEquivalence",
  definition = function(object) {
    cat("******************************",
        "\n",
        object@test,
        "\n",
        rep("-",
            times = nchar(object@test)),
        "\n",
        "Data:                         ",
        object@data$type,
        "\n",
        "H0 (equivalence):             ",
        object@hypotheses$h0,
        "\n",
        "H1 (non-equivalence):         ",
        object@hypotheses$h1,
        "\n",
        "Equivalence interval:         Lower = ",
        formatC(x = object@interval$lower_std,
                digits = 2,
                format = "f"),
        "; Upper = ",
        formatC(x = object@interval$upper_std,
                digits = 2,
                format = "f"),
        " (standardised)",
        "\n",
        "                              Lower = ",
        formatC(x = object@interval$lower_unstd,
                digits = 2,
                format = "f"),
        "; Upper = ",
        formatC(x = object@interval$upper_unstd,
                digits = 2,
                format = "f"),
        " (unstandardised)",
        "\n",
        "Cauchy prior scale:           ",
        formatC(x = object@prior_scale,
                digits = 3,
                format = "f"),
        "\n\n",
        "    BF01 (equivalence) = ",
        if (object@bf > 1 / 1000 && object@bf < 1000) {
          formatC(x = object@bf,
                  digits = 3,
                  format = "f")
        } else {
          formatC(x = object@bf,
                  digits = 3,
                  format = "e")
        },
        "\n",
        "******************************",
        "\n",
        sep = "")
  }
)

setMethod(
  f = "show",
  signature = "baymedrNonInferiority",
  definition = function(object) {
    cat("******************************",
        "\n",
        object@test,
        "\n",
        rep("-",
            times = nchar(object@test)),
        "\n",
        "Data:                         ",
        object@data$type,
        "\n",
        if (object@hypotheses$h1 == "mu_y - mu_x < ni_margin") {
          paste0("H+ (inferiority):             ",
                 object@hypotheses$h0,
                 "\n",
                 "H- (non-inferiority):         ",
                 object@hypotheses$h1)
        },
        if (object@hypotheses$h1 == "mu_y - mu_x > -ni_margin") {
          paste0("H- (inferiority):             ",
                 object@hypotheses$h0,
                 "\n",
                 "H+ (non-inferiority):         ",
                 object@hypotheses$h1)
        },
        "\n",
        "Non-inferiority margin:       ",
        formatC(x = object@ni_margin$ni_mar_std,
                digits = 2,
                format = "f"),
        " (standardised)",
        "\n",
        "                              ",
        formatC(x = object@ni_margin$ni_mar_unstd,
                digits = 2,
                format = "f"),
        " (unstandardised)",
        "\n",
        "Cauchy prior scale:           ",
        formatC(x = object@prior_scale,
                digits = 3,
                format = "f"),
        "\n\n",
        if (object@hypotheses$h1 == "mu_y - mu_x < ni_margin") {
          "    BF-+ (non-inferiority) = "
        },
        if (object@hypotheses$h1 == "mu_y - mu_x > -ni_margin") {
          "    BF+- (non-inferiority) = "
        },
        if (object@bf > 1 / 1000 && object@bf < 1000) {
          formatC(x = object@bf,
                  digits = 3,
                  format = "f")
        } else {
          formatC(x = object@bf,
                  digits = 3,
                  format = "e")
        },
        "\n",
        "******************************",
        "\n",
        sep = "")
  }
)

setMethod(
  f = "show",
  signature = "baymedrCoxProportionalHazards",
  definition = function(object) {
    cat("******************************",
        "\n",
        object@test,
        "\n",
        rep("-",
            times = nchar(object@test)),
        "\n",
        "H0:              ",
        object@hypotheses$h0,
        "\n",
        if (startsWith(x = object@hypotheses$h1,
                       prefix = "beta <")) {
          "H-:              "
        },
        if (startsWith(x = object@hypotheses$h1,
                       prefix = "beta >")) {
          "H+:              "
        },
        if (startsWith(x = object@hypotheses$h1,
                       prefix = "beta !=")) {
          "H1:              "
        },
        object@hypotheses$h1,
        "\n",
        "Normal prior:    Mean = ",
        formatC(x = object@prior$mean,
                digits = 3,
                format = "f"),
        "\n",
        "                 SD = ",
        formatC(x = object@prior$sd,
                digits = 3,
                format = "f"),
        "\n\n",
        if (startsWith(x = object@hypotheses$h1,
                       prefix = "beta <")) {
          "    BF-0 = "
        },
        if (startsWith(x = object@hypotheses$h1,
                       prefix = "beta >")) {
          "    BF+0 = "
        },
        if (startsWith(x = object@hypotheses$h1,
                       prefix = "beta !=")) {
          "    BF10 = "
        },
        if (object@bf > 1 / 1000 && object@bf < 1000) {
          formatC(x = object@bf,
                  digits = 3,
                  format = "f")
        } else {
          formatC(x = object@bf,
                  digits = 3,
                  format = "e")
        },
        "\n",
        "******************************",
        "\n",
        sep = "")
  }
)

setMethod(
  f = "show",
  signature = "baymedrCoxProportionalHazardsMulti",
  definition = function(object) {
    cat("******************************",
        "\n",
        object@test,
        "\n",
        rep("-",
            times = nchar(object@test)),
        "\n",
        "H0:              ",
        object@hypotheses$h0,
        "\n",
        if (startsWith(x = object@hypotheses$h1,
                       prefix = "beta <")) {
          "H-:              "
        },
        if (startsWith(x = object@hypotheses$h1,
                       prefix = "beta >")) {
          "H+:              "
        },
        if (startsWith(x = object@hypotheses$h1,
                       prefix = "beta !=")) {
          "H1:              "
        },
        object@hypotheses$h1,
        "\n",
        "Normal prior:    Mean = ",
        formatC(x = object@prior$mean,
                digits = 3,
                format = "f"),
        "\n",
        "                 SD = ",
        formatC(x = object@prior$sd,
                digits = 3,
                format = "f"),
        "\n\n",
        if (length(object@bf) > 1) {
          paste0(
            if (startsWith(x = object@hypotheses$h1,
                           prefix = "beta <")) {
              "    Median BF-0 = "
            },
            if (startsWith(x = object@hypotheses$h1,
                           prefix = "beta >")) {
              "    Median BF+0 = "
            },
            if (startsWith(x = object@hypotheses$h1,
                           prefix = "beta !=")) {
              "    Median BF10 = "
            },
            if (median(object@bf) > 1 / 1000 && median(object@bf) < 1000) {
              formatC(x = median(object@bf),
                      digits = 3,
                      format = "f")
            } else {
              formatC(x = median(object@bf),
                      digits = 3,
                      format = "e")
            },
            "\n",
            if (startsWith(x = object@hypotheses$h1,
                           prefix = "beta <")) {
              "    MAD SD BF-0 = "
            },
            if (startsWith(x = object@hypotheses$h1,
                           prefix = "beta >")) {
              "    MAD SD BF+0 = "
            },
            if (startsWith(x = object@hypotheses$h1,
                           prefix = "beta !=")) {
              "    MAD SD BF10 = "
            },
            if (1 / qnorm(3 / 4) * median(abs(object@bf - median(object@bf))) > 1 / 1000 &&
                1 / qnorm(3 / 4) * median(abs(object@bf - median(object@bf))) < 1000) {
              formatC(x = 1 / qnorm(3 / 4) * median(abs(object@bf - median(object@bf))),
                      digits = 3,
                      format = "f")
            } else {
              formatC(x = 1 / qnorm(3 / 4) * median(abs(object@bf - median(object@bf))),
                      digits = 3,
                      format = "e")
            }
          )
        } else {
          paste0(
            if (startsWith(x = object@hypotheses$h1,
                           prefix = "beta <")) {
              "    BF-0 = "
            },
            if (startsWith(x = object@hypotheses$h1,
                           prefix = "beta >")) {
              "    BF+0 = "
            },
            if (startsWith(x = object@hypotheses$h1,
                           prefix = "beta !=")) {
              "    BF10 = "
            },
            if (object@bf > 1 / 1000 && object@bf < 1000) {
              formatC(x = object@bf,
                      digits = 3,
                      format = "f")
            } else {
              formatC(x = object@bf,
                      digits = 3,
                      format = "e")
            }
          )
        },
        "\n",
        "******************************",
        "\n",
        sep = "")
  }
)

#' @import rms survival
# This function calculates the hazard ratio and the corresponding x% confidence
# interval for a Cox proportional hazards regression model that is fit using the
# Efron likelihood (cf. Harrell, 2015, p. 477, eq. 20.7).
cox_hr_ci <- function(time,
                      event,
                      group,
                      ci_level = 0.95) {
  mod <- cph(formula = Surv(time, event) ~ group,
             method = "efron",
             eps = 1e-4)
  hr <- exp(coef(object = mod))
  hr_ci <- exp(confint(object = mod,
                       level = ci_level))
  out <- c(hr, hr_ci)
  names(out) <- c("hr", "ci_lower", "ci_upper")
  out
}

# This is the loss function that we want to minimize for the data generation
# process. The loss is based on the hazard ratio and the corresponding
# confidence interval. Using these summary statistics, the loss is defined as
# the log of the weighted and root mean squared deviation between the actual
# summary statistics (i.e., those we want to reach) and the summary statistics
# of the simulated data. It is possible to only provide the hazard ratio and not
# the confidence interval boundaries.
loss <- function(par,
                 cox_hr,
                 time,
                 event,
                 group,
                 cox_hr_ci_level = 0.95) {
  time <- time * exp(par)
  sim_cox_hr <- cox_hr_ci(time = time,
                          event = event,
                          group = group,
                          ci_level = cox_hr_ci_level)
  act <- cox_hr
  sim <- sim_cox_hr
  if (is.na(act[1]) || is.na(sim[1])) {
    stop("The hazard ratio must be provided.",
         call. = FALSE)
  }
  idx <- !is.na(act) & !is.na(sim)
  act <- act[idx]
  sim <- sim[idx]
  w <- c(2, 1, 1)
  w <- w[idx]
  w <- w / sum(w)
  # Mean squared of the scaled weighted errors.
  ms_scaled_weighted_error <- crossprod((sim - act) / act * w) / length(act)
  log(ms_scaled_weighted_error)
}

#' @import stats
bf10_t <- function(t,
                   n1,
                   n2 = NULL,
                   ind_samples = FALSE,
                   prior_loc,
                   prior_scale,
                   prior_df,
                   rel_tol = .Machine$double.eps^0.25) {
  neff <- ifelse(ind_samples,
                 n1 * n2 / (n1 + n2),
                 n1)
  nu <- ifelse(ind_samples,
               n1 + n2 - 2,
               n1 - 1)
  mu_delta <- prior_loc
  gamma <- prior_scale
  kappa <- prior_df
  numerator <- integrate(integrand_t,
                         lower = -Inf,
                         upper = Inf,
                         t = t,
                         n = neff,
                         nu = nu,
                         mu_delta = mu_delta,
                         gamma = gamma,
                         kappa = kappa,
                         rel.tol = rel_tol)$value
  denominator <- dt(x = t,
                    df = nu)
  bf10 <- numerator / denominator
  prior_area_smaller_0 <- pt(q = -mu_delta / gamma,
                             df = kappa)
  post_area_smaller_0 <- cdf_t(x = 0,
                               t = t,
                               n1 = n1,
                               n2 = n2,
                               ind_samples = ind_samples,
                               prior_loc = prior_loc,
                               prior_scale = prior_scale,
                               prior_df = prior_df,
                               rel_tol = rel_tol)
  bf_min1 <- post_area_smaller_0 / prior_area_smaller_0
  bf_plus1 <- (1 - post_area_smaller_0) / (1 - prior_area_smaller_0)
  bf_min0 <- bf_min1 * bf10
  bf_plus0 <- bf_plus1 * bf10
  return(list(
    bf_10 = bf10,
    bf_plus0 = bf_plus0,
    bf_min0 = bf_min0
    ))
}

#' @import stats stringr
cdf_t <- function(x,
                  t,
                  n1,
                  n2 = NULL,
                  ind_samples = FALSE,
                  prior_loc,
                  prior_scale,
                  prior_df,
                  rel_tol = .Machine$double.eps^0.25) {
  out <- integrate(posterior_t,
                   lower = -Inf,
                   upper = x,
                   t = t,
                   n1 = n1,
                   n2 = n2,
                   ind_samples = ind_samples,
                   prior_loc = prior_loc,
                   prior_scale = prior_scale,
                   prior_df = prior_df,
                   rel.tol = rel_tol)$value
  if (out > 1) {
    out <- 1
    warning(str_c(
      "Numerical integration yields a CDF value slightly larger than 1. ",
      "The CDF value has been replaced by 1."
    ),
    call. = FALSE)
  }
  out
}

#' Bayes factor for superiority designs
#'
#' \code{\link{super_bf}} computes a Bayes factor for superiority designs with a
#' continuous dependent variable.
#'
#' The formulation of the null and alternative hypotheses for the superiority
#' design differs depending on whether high or low scores on the dependent
#' variable represent superiority. In both cases, the null hypothesis (i.e., H0)
#' states that the population means of the experimental group and the control
#' group are equivalent. In the case where high scores correspond to
#' superiority, the alternative hypothesis states that the population mean of
#' the experimental group is higher than the population mean of the control
#' group. Thus, the alternative hypothesis goes in the positive direction (i.e.,
#' H+). In turn, in the case where low scores correspond to superiority, the
#' alternative hypothesis states that the population mean of the experimental
#' group is lower than the population mean of the control group. Thus, the
#' alternative hypothesis goes in the negative direction (i.e., H-). The
#' dependent variable must be continuous.
#'
#' Since the main goal of \code{\link{super_bf}} is to establish superiority,
#' the resulting Bayes factor quantifies evidence in favor of the alternative
#' hypothesis. In the case where low values represent superiority we have BF-0,
#' whereas in the case where high values represent superiority we have BF+0.
#' Evidence for the null hypothesis can easily be calculated by taking the
#' reciprocal of the original Bayes factor (i.e., BF0- = 1 / BF-0 and BF0+ = 1 /
#' BF+0). Quantification of evidence in favor of the null hypothesis is
#' logically sound and legitimate within the Bayesian framework (see e.g., van
#' Ravenzwaaij et al., 2019).
#'
#' \code{\link{super_bf}} can be utilized to calculate a Bayes factor based on
#' raw data (i.e., if arguments \code{x} and \code{y} are defined) or summary
#' statistics (i.e., if arguments \code{n_x}, \code{n_y}, \code{mean_x}, and
#' \code{mean_y} are defined). In the latter case, the user has the freedom to
#' supply values either for the arguments \code{sd_x} and \code{sd_y}
#' \strong{OR} \code{ci_margin} and \code{ci_level}. Arguments with 'x' as a
#' name or suffix correspond to the control group, whereas arguments with 'y' as
#' a name or suffix correspond to the experimental group (i.e., the group for
#' which we seek to establish superiority).
#'
#' For the calculation of the Bayes factor, a Cauchy prior density centered on 0
#' is chosen for the effect size under the alternative hypothesis. The standard
#' Cauchy distribution, with a location parameter of 0 and a scale parameter of
#' 1, resembles a standard Normal distribution, except that the Cauchy
#' distribution has less mass at the center but heavier tails (Liang et al.,
#' 2008; Rouder et al., 2009). The argument \code{prior_scale} specifies the
#' width of the Cauchy prior, which corresponds to half of the interquartile
#' range. Thus, by adjusting the Cauchy prior scale with \code{prior_scale},
#' different ranges of expected effect sizes can be emphasized. The default
#' prior scale is set to r = 1 / sqrt(2).
#'
#' \code{\link{super_bf}} creates an S4 object of class
#' \linkS4class{baymedrSuperiority}, which has multiple slots/entries (e.g.,
#' type of data, prior scale, Bayes factor, etc.; see Value). If it is desired
#' to store or extract solely the Bayes factor, the user can do this with
#' \code{\link{get_bf}}, by setting the S4 object as an argument (see Examples).
#'
#' @param x A numeric vector of observations for the control group.
#' @param y A numeric vector of observations for the experimental group.
#' @param n_x A numeric vector of length one, specifying the sample size of the
#'   control group.
#' @param n_y A numeric vector of length one, specifying the sample size of the
#'   experimental group.
#' @param mean_x A numeric vector of length one, specifying the mean of the
#'   dependent variable in the control group.
#' @param mean_y A numeric vector of length one, specifying the mean of the
#'   dependent variable in the experimental group.
#' @param sd_x A numeric vector of length one, specifying the standard deviation
#'   of the dependent variable in the control group. Only \code{sd_x} and
#'   \code{sd_y} \strong{OR} \code{ci_margin} and \code{ci_level} should be
#'   defined (see Details).
#' @param sd_y A numeric vector of length one, specifying the standard deviation
#'   of the dependent variable in the experimental group. Only \code{sd_x} and
#'   \code{sd_y} \strong{OR} \code{ci_margin} and \code{ci_level} should be
#'   defined (see Details).
#' @param ci_margin A numeric vector of length one, specifying the margin of the
#'   confidence interval (i.e., the width of the confidence interval divided by
#'   2) of the mean difference on the dependent variable between the
#'   experimental and control groups. The value should be a positive number Only
#'   \code{sd_x} and \code{sd_y} \strong{OR} \code{ci_margin} and
#'   \code{ci_level} should be defined (see Details).
#' @param ci_level A numeric vector of length one, specifying the confidence
#'   level of \code{ci_margin}. The value must be between 0 and 1 (e.g., 0.95
#'   for a 95\% confidence interval). Only \code{sd_x} and \code{sd_y}
#'   \strong{OR} \code{ci_margin} and \code{ci_level} should be defined (see
#'   Details).
#' @param prior_scale A numeric vector of length one, specifying the scale of
#'   the Cauchy prior distribution for the effect size under the alternative
#'   hypothesis (see Details). The default value is r = 1 / sqrt(2).
#' @param direction A character vector of length one, specifying the direction
#'   of superior scores. 'low' indicates that low scores on the measure of
#'   interest correspond to a superior outcome and 'high' (the default)
#'   indicates that high scores on the measure of interest correspond to a
#'   superior outcome (see Details).
#'
#' @return An S4 object of class \linkS4class{baymedrSuperiority} is returned.
#'   Contained are a description of the model and the resulting Bayes factor:
#'   \itemize{ \item test: The type of analysis \item hypotheses: A statement of
#'   the hypotheses \itemize{ \item h0: The null hypothesis \item h1: The
#'   alternative hypothesis} \item data: A description of the data \itemize{
#'   \item type: The type of data ('raw' when arguments \code{x} and \code{y}
#'   are used or 'summary' when arguments \code{n_x}, \code{n_y}, \code{mean_x},
#'   \code{mean_y}, \code{sd_x}, and \code{sd_y} (or \code{ci_margin} and
#'   \code{ci_level} instead of \code{sd_x} and \code{sd_y}) are used) \item
#'   ...: values for the arguments used, depending on 'raw' or 'summary'} \item
#'   prior_scale: The scale of the Cauchy prior distribution \item bf: The
#'   resulting Bayes factor } A summary of the model is shown by printing the
#'   object.
#'
#' @export
#' @import stats stringr
#'
#' @references
#'   Gronau, Q. F., Ly, A., & Wagenmakers, E.-J. (2020). Informed Bayesian
#'   t-tests. \emph{The American Statistician}, \emph{74}, 137-143.
#'
#'   Liang, F., Paulo, R., Molina, G., Clyde, M. A., & Berger, J. O. (2008).
#'   Mixtures of g priors for Bayesian variable selection. \emph{Journal of the
#'   American Statistical Association}, \emph{103}, 410-423.
#'
#'   Rouder, J. N., Speckman, P. L., Sun, D., Morey, R. D., & Iverson, G.
#'   (2009). Bayesian t tests for accepting and rejecting the null hypothesis.
#'   \emph{Psychonomic Bulletin & Review}, \emph{16}, 225-237.
#'
#'   van Ravenzwaaij, D., Monden, R., Tendeiro, J. N., & Ioannidis, J. P. A.
#'   (2019). Bayes factors for superiority, non-inferiority, and equivalence
#'   designs. \emph{BMC Medical Research Methodology}, \emph{19}, 71.
#'
#' @examples
#' ## super_bf using raw data:
#'
#' # Assign model to variable.
#' super_raw <- super_bf(x = rnorm(100, 10, 15),
#'                       y = rnorm(130, 13, 10))
#'
#' # Extract Bayes factor from model.
#' get_bf(super_raw)
#'
#' # ----------
#' # ----------
#'
#' ## super_bf using summary statistics with data from Skjerven et al. (2013).
#' ## EXAMPLE 1
#'
#' # Assign model to variable.
#' super_sum_ex1 <- super_bf(n_x = 201,
#'                           n_y = 203,
#'                           mean_x = 68.1,
#'                           mean_y = 63.6,
#'                           ci_margin = (15.5 - (-6.5)) / 2,
#'                           ci_level = 0.95,
#'                           direction = "low")
#'
#' # Extract Bayes factor from model.
#' get_bf(super_sum_ex1)
#'
#' # ----------
#'
#' ## super_bf using summary statistics with data from Skjerven et al. (2013).
#' ## EXAMPLE 2
#'
#' # Assign model to variable.
#' super_sum_ex2 <- super_bf(n_x = 200,
#'                           n_y = 204,
#'                           mean_x = 47.6,
#'                           mean_y = 61.3,
#'                           ci_margin = (24.4 - 2.9) / 2,
#'                           ci_level = 0.95,
#'                           direction = "low")
#'
#' # Extract Bayes factor from model.
#' get_bf(super_sum_ex2)
super_bf <- function(x = NULL,
                     y = NULL,
                     n_x = NULL,
                     n_y = NULL,
                     mean_x = NULL,
                     mean_y = NULL,
                     sd_x = NULL,
                     sd_y = NULL,
                     ci_margin = NULL,
                     ci_level = NULL,
                     prior_scale = 1 / sqrt(2),
                     direction = "high") {
  if (any(!is.null(x),
          !is.null(y)) && any(!is.null(n_x),
                              !is.null(n_y),
                              !is.null(mean_x),
                              !is.null(mean_y),
                              !is.null(sd_x),
                              !is.null(sd_y),
                              !is.null(ci_margin),
                              !is.null(ci_level))) {
    stop(str_c(
      "Only 'x' and 'y' OR 'n_x', 'n_y', 'mean_x', 'mean_y', 'sd_x', and ",
      "'sd_y' (or 'ci_margin' and 'ci_level' instead of 'sd_x' and 'sd_y') ",
      "must be defined."
    ),
    call. = FALSE)
  }
  if (xor(!is.null(x),
          !is.null(y))) {
    stop("Both 'x' and 'y' must be defined.",
         call. = FALSE)
  }
  if (any(!is.null(n_x),
          !is.null(n_y),
          !is.null(mean_x),
          !is.null(mean_y))) {
    if (any(is.null(n_x),
            is.null(n_y),
            is.null(mean_x),
            is.null(mean_y)) ||
        ((is.null(sd_x) || is.null(sd_y)) &&
         (is.null(ci_margin) || is.null(ci_level)))) {
      stop(str_c(
        "All 'n_x', 'n_y', 'mean_x', 'mean_y', 'sd_x', and 'sd_y' (or ",
        "'ci_margin' and 'ci_level' instead of 'sd_x' and 'sd_y') must be ",
        "defined."
      ),
      call. = FALSE)
    }
    if (!xor(!is.null(sd_x) && !is.null(sd_y),
             !is.null(ci_margin) && !is.null(ci_level))) {
      stop(
        "Only 'sd_x' and 'sd_y' OR 'ci_margin' and 'ci_level' must be defined.",
        call. = FALSE
      )
    }
  }
  if (all(!is.null(n_x),
          !is.null(n_y),
          !is.null(mean_x),
          !is.null(mean_y)) && (xor(
            !is.null(sd_x) && !is.null(sd_y),
            !is.null(ci_margin) && !is.null(ci_level)
          ))) {
    if (!is.null(ci_level) && (length(ci_level) > 1 || ci_level <= 0 ||
                               ci_level >= 1 || !is.numeric(ci_level))) {
      stop("'ci_level' must be a single numeric value between 0 and 1.",
           call. = FALSE)
    }
    data <- list(type = "summary data",
                 data = list(n_x = n_x,
                             n_y = n_y,
                             mean_x = mean_x,
                             mean_y = mean_y,
                             sd_x = sd_x,
                             sd_y = sd_y,
                             ci_margin = ci_margin,
                             ci_level = ci_level))
  }
  if (!is.null(x) && !is.null(y)) {
    if (any(is.na(x)) || any(is.na(y))) {
      stop("'x' and 'y' must not contain missing values.",
           call. = FALSE)
    }
    if (any(is.infinite(x)) || any(is.infinite(y))) {
      stop("'x' and 'y' must not contain infinite values.",
           call. = FALSE)
    }
    if (!is.numeric(x) || !is.numeric(y)) {
      stop("'x' and 'y' must be numeric vectors.",
           call. = FALSE)
    }
    n_x <- length(x)
    n_y <- length(y)
    mean_x <- mean(x)
    mean_y <- mean(y)
    sd_x <- sd(x)
    sd_y <- sd(y)
    data <- list(type = "raw data",
                 data = list(x = x,
                             y = y))
  }
  if (!is.numeric(prior_scale) || length(prior_scale) > 1) {
    stop("'prior_scale' must be a single numeric value.",
         call. = FALSE)
  }
  if (!is.character(direction) || length(direction) > 1) {
    stop("'direction' must be a single character value.",
         call. = FALSE)
  }
  if (!is.null(sd_x) && !is.null(sd_y)) {
    sd_pooled <- sqrt(((n_x - 1) * sd_x ^ 2 + (n_y - 1) * sd_y ^ 2) /
                        (n_x + n_y - 2))
    se <- sd_pooled * sqrt(1 / n_x + 1 / n_y)
  } else {
    perc <- 1 - ((1 - ci_level) / 2)
    se <- ci_margin / qt(p = perc,
                         df = n_x + n_y - 2)
  }
  t_stat <- (mean_y - mean_x) / se
  res <- bf10_t(t = t_stat,
                n1 = n_x,
                n2 = n_y,
                ind_samples = TRUE,
                prior_loc = 0,
                prior_scale = prior_scale,
                prior_df = 1)
  if (str_detect(direction,
                 "low")) {
    bf <- res$bf_min0
    h1 <- "mu_y < mu_x"
  } else if (str_detect(direction,
                        "high")) {
    bf <- res$bf_plus0
    h1 <- "mu_y > mu_x"
  } else {
    stop("'direction' must be one of 'low' or 'high'.",
         call. = FALSE)
  }
  test <- "Superiority analysis"
  h0 <- "mu_y == mu_x"
  hypotheses <- list(h0 = h0,
                     h1 = h1)
  new(Class = "baymedrSuperiority",
      test = test,
      data = data,
      hypotheses = hypotheses,
      prior_scale = prior_scale,
      bf = bf)
}

#' @import stats
integrand_t <- function(delta,
                        t,
                        n,
                        nu,
                        mu_delta,
                        gamma,
                        kappa) {
  suppressWarnings(
    dt(
      x = t,
      df = nu,
      ncp = sqrt(n) * delta) * 1 / gamma * dt(
        (delta - mu_delta) / gamma,
        df = kappa))
}

#' baymedr: Computation of Bayes Factors for Common Biomedical Designs
#'
#' \code{baymedr} provides functions for the computation of Bayes factors for
#' common biomedical research designs.
#'
#' \tabular{ll}{ Package: \tab baymedr \cr Type: \tab Package \cr Version: \tab
#' 0.2 \cr License: \tab GPL-3 \cr Date: \tab 2022-09-01 \cr}
#'
#' @docType package
#' @name baymedr-package
#' @aliases baymedr-package baymedr
#' @author \strong{Maintainer}: Maximilian Linde --
#'   \email{maximilian.linde.92@@gmail.com}
#'
#'   Authors: \itemize{ \item Maximilian Linde (aut, cre) \item Don van
#'   Ravenzwaaij (aut) \item Jorge N. Tendeiro (aut)
#'   \item Quentin F. Gronau (ctb) }
#'
#' @seealso Useful links: \itemize{ \item
#'   \url{https://github.com/maxlinde/baymedr} \item Report bugs at
#'   \url{https://github.com/maxlinde/baymedr/issues}}
"_PACKAGE"

#' S4 classes to represent different models
#'
#' The S4 classes \linkS4class{baymedrSuperiority},
#' \linkS4class{baymedrEquivalence}, \linkS4class{baymedrNonInferiority},
#' \linkS4class{baymedrCoxProportionalHazards}, and
#' \linkS4class{baymedrCoxProportionalHazardsMulti} represent models for the
#' superiority (\code{\link{super_bf}}), equivalence (\code{\link{equiv_bf}}),
#' non-inferiority (\code{\link{infer_bf}}), and Cox proportional hazards
#' (\code{\link{coxph_bf}}) models, respectively.
#'
#' @slot test Type of test that was conducted.
#' @slot hypotheses The hypotheses that are tested.
#' @slot data The type of data that was used.
#' @slot prior_scale The Cauchy prior scale that was used.
#' @slot bf The resulting Bayes factor.
#' @slot interval The equivalence interval in case of \code{\link{equiv_bf}}.
#' @slot ni_margin The non-inferiority margin in case of \code{\link{infer_bf}}.
#' @slot prior The mean and standard deviation of the Normal prior in case of
#'   \code{\link{coxph_bf}}.
#'
#' @name model-classes
#'
#' @import methods
NULL

#' @rdname model-classes
setClass(Class = "baymedrSuperiority",
         slots = c(
           test = "character",
           hypotheses = "list",
           data = "list",
           prior_scale = "numeric",
           bf = "numeric"
         ),
         prototype = list(
           test = NA_character_,
           hypotheses = list(),
           data = list(),
           prior_scale = NA_real_,
           bf = NA_real_
         ))

#' @rdname model-classes
setClass(Class = "baymedrEquivalence",
         slots = c(
           test = "character",
           hypotheses = "list",
           interval = "list",
           data = "list",
           prior_scale = "numeric",
           bf = "numeric"
         ),
         prototype = list(
           test = NA_character_,
           hypotheses = list(),
           interval = list(),
           data = list(),
           prior_scale = NA_real_,
           bf = NA_real_
         ))

#' @rdname model-classes
setClass(Class = "baymedrNonInferiority",
         slots = c(
           test = "character",
           hypotheses = "list",
           ni_margin = "list",
           data = "list",
           prior_scale = "numeric",
           bf = "numeric"
         ),
         prototype = list(
           test = NA_character_,
           hypotheses = list(),
           ni_margin = list(),
           data = list(),
           prior_scale = NA_real_,
           bf = NA_real_
         ))

#' @rdname model-classes
setClass(Class = "baymedrCoxProportionalHazards",
         slots = c(
           test = "character",
           hypotheses = "list",
           prior = "list",
           bf = "numeric"
         ),
         prototype = list(
           test = NA_character_,
           hypotheses = list(),
           prior = list(),
           bf = NA_real_
         ))

#' @rdname model-classes
setClass(Class = "baymedrCoxProportionalHazardsMulti",
         slots = c(
           test = "character",
           hypotheses = "list",
           prior = "list",
           bf = "numeric"
         ),
         prototype = list(
           test = NA_character_,
           hypotheses = list(),
           prior = list(),
           bf = NA_real_
         ))

#' Bayes factor for Cox proportional hazards regression
#'
#' \code{\link{coxph_bf}} computes a Bayes factor for Cox proportional hazards
#' regression models with one dichotomous independent variable.
#'
#' The Cox proportional hazards model has the following hypotheses: The null
#' hypothesis (i.e., H0) states that the population hazard ratio between the
#' experimental (e.g., a new medication) and the control group (e.g., a placebo
#' or an already existing medication) is equal to 1 (i.e., beta = 0). The
#' alternative hypothesis can be two-sided or one-sided (either negative or
#' positive).
#'
#' Since the main goal of \code{\link{coxph_bf}} is to establish that the
#' hazard ratio is not equal to 1, the resulting Bayes factor quantifies
#' evidence in favor of the alternative hypothesis. For a two-sided alternative
#' hypothesis, we have BF10; for a negative one-sided alternative hypothesis, we
#' have BF-0; and for a positive one-sided alternative hypothesis, we have BF+0.
#' Evidence for the null hypothesis can easily be calculated by taking the
#' reciprocal of the original Bayes factor (i.e., BF01 = 1 / BF10).
#' Quantification of evidence in favor of the null hypothesis is logically sound
#' and legitimate within the Bayesian framework (see e.g., van Ravenzwaaij et
#' al., 2019).
#'
#' For the calculation of the Bayes factor, a Normal prior density is chosen for
#' beta under the alternative hypothesis. The arguments \code{prior_mean} and
#' \code{prior_sd} specify the mean and standard deviation of the Normal prior,
#' respectively. By adjusting the Normal prior, different ranges of expected
#' effect sizes can be emphasized. The default is a Normal prior with a mean of
#' 0 and a standard deviation of 1.
#'
#' Note that at the moment the model specifications are limited. That is, it is
#' only possible to have a single dichotomous independent variable. Further, at
#' the moment only a Normal prior is supported. Lastly, only the Efron partial
#' likelihood and not the many other options are supported.
#'
#' \code{\link{coxph_bf}} creates an S4 object of class
#' \linkS4class{baymedrCoxProportionalHazards}, which has multiple slots/entries
#' (e.g., prior, Bayes factor, etc.; see Value). If it is desired to store or
#' extract solely the Bayes factor, the user can do this with
#' \code{\link{get_bf}}, by setting the S4 object as an argument (see Examples).
#'
#' @param data A data.frame or a list resulting from calling
#'   \code{coxph_data_sim}. In the former case, the first column represents the
#'   survival/censoring times, the second column indicates whether an event
#'   (e.g., death) has happened (1) or not (0), and the third column represents
#'   the dummy-coded independent variable, where the control group is coded with
#'   0 and the experimental group with 1.
#' @param null_value The value of the point null hypothesis for the beta
#'   coefficient. The default is a null value of 0.
#' @param alternative A string specifying whether the alternative hypothesis is
#'   two-sided ("two.sided"; the default) or one-sided ("one.sided").
#' @param direction A string specifying the direction of the one-sided
#'   alternative hypothesis. This is ignored if
#'   \code{alternative = "two.sided"}. Possible options are "low" and "high".
#' @param prior_mean Mean of the Normal prior for the beta parameter. The
#'   default is a mean of 0.
#' @param prior_sd Standard deviation of the Normal prior for the beta
#'   parameter. The default is a standard deviation of 1.
#'
#' @return An S4 object of class \linkS4class{baymedrCoxProportionalHazards} is
#'   returned. Contained are a description of the model and the resulting Bayes
#'   factor: \itemize{ \item test: The type of analysis \item hypotheses: A
#'   statement of the hypotheses \itemize{ \item h0: The null hypothesis
#'   \item h1: The alternative hypothesis} \item prior: The parameters of the
#'   Normal prior on beta \item bf: The resulting Bayes factor } A summary of
#'   the model is shown by printing the object.
#'
#' @seealso \code{\link{coxph_data_sim}}.
#'
#' @export
#' @import stringr
#'
#' @references
#'   Harrell, F. R. (2015). Regression modeling strategies: With applications to
#'   linear models, logistic regression, and survival analysis (2nd ed.).
#'   Springer.
#'
#'   van Ravenzwaaij, D., Monden, R., Tendeiro, J. N., & Ioannidis, J. P. A.
#'   (2019). Bayes factors for superiority, non-inferiority, and equivalence
#'   designs. \emph{BMC Medical Research Methodology}, \emph{19}, 71.
#'
#' @examples
#' # Load aml dataset from the survival R package.
#' data <- survival::aml
#' data$x <- ifelse(test = data$x == "Maintained",
#'                  yes = 0,
#'                  no = 1)
#' names(data) <- c("time", "event", "group")
#'
#' # Assign model to variable.
#' coxph_mod <- coxph_bf(data = data,
#'                       null_value = 0,
#'                       alternative = "one.sided",
#'                       direction = "high",
#'                       prior_mean = 0,
#'                       prior_sd = 1)
#'
#' # Extract Bayes factor from variable.
#' get_bf(coxph_mod)
coxph_bf <- function(data,
                     null_value = 0,
                     alternative = "two.sided",
                     direction = NULL,
                     prior_mean = 0,
                     prior_sd = 1) {
  if (!inherits(x = data,
                what = c("data.frame", "list"))) {
    stop(str_c("'data' must be either a single data.frame or a list resulting ",
               "from coxph_data_sim()."),
         call. = FALSE)
  }
  if (length(null_value) != 1 || !is.numeric(null_value)) {
    stop("'null_value' must be a single numeric value.",
         call. = FALSE)
  }
  if (length(alternative) != 1 ||
      !(alternative %in% c("one.sided", "two.sided"))) {
    stop("'alternative' must be one of one.sided or two.sided.",
         call. = FALSE)
  }
  if (alternative == "two.sided") {
    if (!is.null(direction)) {
      stop("When 'alternative' is two.sided, 'direction' must be NULL.",
           call. = FALSE)
    }
  } else {
    if (length(direction) != 1 || is.null(direction) ||
        !(direction %in% c("low", "high"))) {
      stop("When 'alternative' is one.sided, 'direction' must be low or high.",
           call. = FALSE)
    }
  }
  if (length(prior_mean) != 1 || !is.numeric(prior_mean) ||
      length(prior_sd) != 1 || !is.numeric(prior_sd) || prior_sd < 0) {
    stop(str_c("'prior_mean' and 'prior_sd' must be single numeric values. ",
               "'prior_sd' must be positive."),
         call. = FALSE)
  }
  test <- "Cox proportional hazards analysis"
  h0 <- paste0("beta == ", null_value)
  if (alternative == "two.sided") {
    h1 <- paste0("beta != ", null_value)
  } else {
    if (direction == "low") {
      h1 <- paste0("beta < ", null_value)
    } else {
      h1 <- paste0("beta > ", null_value)
    }
  }
  hypotheses <- list(h0 = h0,
                     h1 = h1)
  prior <- list(mean = prior_mean,
                sd = prior_sd)
  if (inherits(x = data,
               what = "data.frame")) {
    time <- data[[1]]
    event <- data[[2]]
    group <- data[[3]]
    if (!is.numeric(time) || any(time < 0, na.rm = TRUE)) {
      stop("The first column in 'data' must be a non-negative numeric vector.",
           call. = FALSE)
    }
    if (!is.numeric(event) || !all(event %in% c(0:1, NA))) {
      stop(str_c("The second column in 'data' must be a numeric vector ",
                 "containing only the values 0 and 1."),
           call. = FALSE)
    }
    if (!is.numeric(group) || !all(group %in% c(0:1, NA))) {
      stop(str_c("The third column in 'data' must be a numeric vector ",
                 "containing only the values 0 and 1."),
           call. = FALSE)
    }
    if (any(is.na(data))) {
      stop("'data' must not contain any missing values.",
           call. = FALSE)
    }
    bf <- bf10(time = time,
               event = event,
               group = group,
               null_value = null_value,
               alternative = alternative,
               direction = direction,
               prior_mean = prior_mean,
               prior_sd = prior_sd)
    new(Class = "baymedrCoxProportionalHazards",
        test = test,
        hypotheses = hypotheses,
        prior = prior,
        bf = bf)
  } else {
    n_elem <- length(data)
    bf <- vector(mode = "numeric",
                 length = n_elem)
    for (i in 1:n_elem) {
      tmp_data <- data[[i]][["data"]]
      time <- tmp_data[[1]]
      event <- tmp_data[[2]]
      group <- tmp_data[[3]]
      if (!is.numeric(time) || any(time < 0, na.rm = TRUE)) {
        stop(str_c("The first column in 'data' must be a non-negative numeric ",
                   "vector."),
             call. = FALSE)
      }
      if (!is.numeric(event) || !all(event %in% c(0:1, NA))) {
        stop(str_c("The second column in 'data' must be a numeric vector ",
                   "containing only the values 0 and 1."),
             call. = FALSE)
      }
      if (!is.numeric(group) || !all(group %in% c(0:1, NA))) {
        stop(str_c("The third column in 'data' must be a numeric vector ",
                   "containing only the values 0 and 1."),
             call. = FALSE)
      }
      if (any(is.na(data))) {
        stop("'data' must not contain any missing values.",
             call. = FALSE)
      }
      bf[i] <- bf10(time = time,
                    event = event,
                    group = group,
                    null_value = null_value,
                    alternative = alternative,
                    direction = direction,
                    prior_mean = prior_mean,
                    prior_sd = prior_sd)
    }
    new(Class = "baymedrCoxProportionalHazardsMulti",
        test = test,
        hypotheses = hypotheses,
        prior = prior,
        bf = bf)
  }
}

context("test-coxph_bf")

data <- survival::aml
names(data) <- c("time", "event", "group")
data$group <- ifelse(test = survival::aml$x == "Maintained",
                     yes = 0,
                     no = 1)

sim_data <- coxph_data_sim(n_data = 3,
                           ns_c = 20,
                           ns_e = 56,
                           ne_c = 18,
                           ne_e = 40,
                           cox_hr = c(0.433, 0.242, 0.774),
                           cox_hr_ci_level = 0.95,
                           maxit = 25)


test_that("coxph_bf yields correct S4 class", {
  expect_is(
    coxph_bf(data = data),
    "baymedrCoxProportionalHazards"
  )
  expect_is(
    coxph_bf(data = sim_data),
    "baymedrCoxProportionalHazardsMulti"
  )
})

test_that("coxph_bf yields numeric Bayes factor", {
  expect_true(
    is.numeric(coxph_bf(data = data)@bf)
  )
  expect_true(
    is.numeric(coxph_bf(data = sim_data)@bf)
  )
})

test_that("coxph_bf gives correct error messages", {
  expect_error(
    coxph_bf(data = factor(sample(1:5, 100, TRUE))),
    str_c("'data' must be either a single data.frame or a list resulting ",
          "from coxph_data_sim()."),
    fixed = TRUE
  )
  expect_error(
    coxph_bf(data = data.frame(
      time = sample(x = letters,
                    size = 100,
                    replace = TRUE),
      event = sample(x = 0:1,
                     size = 100,
                     replace = TRUE),
      group = sample(x = 0:1,
                     size = 100,
                     replace = TRUE)
    )),
    "The first column in 'data' must be a non-negative numeric vector.",
    fixed = TRUE
  )
  expect_error(
    coxph_bf(data = data.frame(
      time = rnorm(n = 100,
                   mean = -100),
      event = sample(x = 0:1,
                     size = 100,
                     replace = TRUE),
      group = sample(x = 0:1,
                     size = 100,
                     replace = TRUE)
    )),
    "The first column in 'data' must be a non-negative numeric vector.",
    fixed = TRUE
  )
  expect_error(
    coxph_bf(data = data.frame(
      time = runif(n = 100,
                   min = 0,
                   max = 100),
      event = sample(x = letters,
                     size = 100,
                     replace = TRUE),
      group = sample(x = 0:1,
                     size = 100,
                     replace = TRUE)
    )),
    str_c("The second column in 'data' must be a numeric vector containing ",
          "only the values 0 and 1."),
    fixed = TRUE
  )
  expect_error(
    coxph_bf(data = data.frame(
      time = runif(n = 100,
                   min = 0,
                   max = 100),
      event = sample(x = 0:5,
                     size = 100,
                     replace = TRUE),
      group = sample(x = 0:1,
                     size = 100,
                     replace = TRUE)
    )),
    str_c("The second column in 'data' must be a numeric vector containing ",
          "only the values 0 and 1."),
    fixed = TRUE
  )
  expect_error(
    coxph_bf(data = data.frame(
      time = runif(n = 100,
                   min = 0,
                   max = 100),
      event = sample(x = 0:1,
                     size = 100,
                     replace = TRUE),
      group = sample(x = letters,
                     size = 100,
                     replace = TRUE)
    )),
    str_c("The third column in 'data' must be a numeric vector containing ",
          "only the values 0 and 1."),
    fixed = TRUE
  )
  expect_error(
    coxph_bf(data = data.frame(
      time = runif(n = 100,
                   min = 0,
                   max = 100),
      event = sample(x = 0:1,
                     size = 100,
                     replace = TRUE),
      group = sample(x = 0:5,
                     size = 100,
                     replace = TRUE)
    )),
    str_c("The third column in 'data' must be a numeric vector containing ",
          "only the values 0 and 1."),
    fixed = TRUE
  )
  expect_error(
    coxph_bf(data = data.frame(
      time = runif(n = 100,
                   min = 0,
                   max = 100),
      event = sample(x = 0:1,
                     size = 100,
                     replace = TRUE),
      group = sample(x = 0:1,
                     size = 100,
                     replace = TRUE)
    ),
    null_value = "a"),
    "'null_value' must be a single numeric value.",
    fixed = TRUE
  )
  expect_error(
    coxph_bf(data = data.frame(
      time = runif(n = 100,
                   min = 0,
                   max = 100),
      event = sample(x = 0:1,
                     size = 100,
                     replace = TRUE),
      group = sample(x = 0:1,
                     size = 100,
                     replace = TRUE)
    ),
    null_value = -1:1),
    "'null_value' must be a single numeric value.",
    fixed = TRUE
  )
  expect_error(
    coxph_bf(data = data.frame(
      time = runif(n = 100,
                   min = 0,
                   max = 100),
      event = sample(x = 0:1,
                     size = 100,
                     replace = TRUE),
      group = sample(x = 0:1,
                     size = 100,
                     replace = TRUE)
    ),
    alternative = "abc"),
    "'alternative' must be one of one.sided or two.sided.",
    fixed = TRUE
  )
  expect_error(
    coxph_bf(data = data.frame(
      time = runif(n = 100,
                   min = 0,
                   max = 100),
      event = sample(x = 0:1,
                     size = 100,
                     replace = TRUE),
      group = sample(x = 0:1,
                     size = 100,
                     replace = TRUE)
    ),
    alternative = "two.sided",
    direction = -1),
    "When 'alternative' is two.sided, 'direction' must be NULL.",
    fixed = TRUE
  )
  expect_error(
    coxph_bf(data = data.frame(
      time = runif(n = 100,
                   min = 0,
                   max = 100),
      event = sample(x = 0:1,
                     size = 100,
                     replace = TRUE),
      group = sample(x = 0:1,
                     size = 100,
                     replace = TRUE)
    ),
    alternative = "one.sided",
    direction = NULL),
    "When 'alternative' is one.sided, 'direction' must be low or high.",
    fixed = TRUE
  )
  expect_error(
    coxph_bf(data = data.frame(
      time = runif(n = 100,
                   min = 0,
                   max = 100),
      event = sample(x = 0:1,
                     size = 100,
                     replace = TRUE),
      group = sample(x = 0:1,
                     size = 100,
                     replace = TRUE)
    ),
    prior_sd = 1:3),
    str_c("'prior_mean' and 'prior_sd' must be single numeric values. ",
          "'prior_sd' must be positive."),
    fixed = TRUE
  )
  expect_error(
    coxph_bf(data = data.frame(
      time = runif(n = 100,
                   min = 0,
                   max = 100),
      event = sample(x = 0:1,
                     size = 100,
                     replace = TRUE),
      group = sample(x = 0:1,
                     size = 100,
                     replace = TRUE)
    ),
    prior_sd = -1),
    str_c("'prior_mean' and 'prior_sd' must be single numeric values. ",
          "'prior_sd' must be positive."),
    fixed = TRUE
  )
  expect_error(
    coxph_bf(data = data.frame(
      time = c(runif(n = 99,
                     min = 0,
                     max = 100), NA),
      event = sample(x = 0:1,
                     size = 100,
                     replace = TRUE),
      group = sample(x = 0:1,
                     size = 100,
                     replace = TRUE)
    )),
    "'data' must not contain any missing values.",
    fixed = TRUE
  )
})

log_likelihood <- function(beta,
                           time,
                           event,
                           group) {
  y <- time
  delta <- event
  x <- group
  t <- unique(sort(y[delta == 1]))
  ll <- 0
  for (j in 1:(length(t))) {
    n_set <- which(y == t[j] & delta == 1)
    n <- length(n_set)
    s_set <- which(y >= t[j])
    a <- x[n_set]
    f <- a * beta
    v <- x[s_set]
    q <- v * beta
    tmp_1 <- sum(a) * beta
    tmp_2 <- 0
    if (any(q > 0) | all(q < 0)) {
      z <- -max(v) * beta
    } else {
      z <- 0
    }
    for (i in 1:n) {
      tmp_2a <- sum(exp(q + z))
      tmp_2b <- (i - 1) / n
      tmp_2c <- sum(exp(f + z))
      tmp_2 <- tmp_2 + log(tmp_2a - tmp_2b * tmp_2c)
    }
    ll <- ll + tmp_1 - (tmp_2 - n * z)
  }
  ll
}
log_likelihood <- Vectorize(FUN = log_likelihood,
                            vectorize.args = "beta")

bf10 <- function(time,
                 event,
                 group,
                 null_value = 0,
                 alternative = "two.sided",
                 direction = NULL,
                 prior_mean = 0,
                 prior_sd = 1) {
  log_lik <- log_likelihood(beta = null_value,
                            time = time,
                            event = event,
                            group = group)
  if (alternative == "two.sided") {
    m_lower <- -100
    m_upper <- 100
    i_lower <- -Inf
    i_upper <- Inf
    prior_adj <- log(1)
  } else if (alternative == "one.sided" & direction == "low") {
    m_lower <- -100
    m_upper <- null_value
    i_lower <- -Inf
    i_upper <- null_value
    prior_adj <- pnorm(q = null_value,
                       mean = prior_mean,
                       sd = prior_sd,
                       lower.tail = TRUE,
                       log.p = TRUE)
  } else if (alternative == "one.sided" & direction == "high") {
    m_lower <- null_value
    m_upper <- 100
    i_lower <- null_value
    i_upper <- Inf
    prior_adj <- pnorm(q = null_value,
                       mean = prior_mean,
                       sd = prior_sd,
                       lower.tail = FALSE,
                       log.p = TRUE)
  } else {
    stop("Incorrect arguments for 'alternative' and 'direction' used.",
         call. = FALSE)
  }
  m <- optim(par = null_value,
             fn = function(x) {
               log_likelihood(beta = x,
                              time = time,
                              event = event,
                              group = group) + dnorm(x = x,
                                                     mean = prior_mean,
                                                     sd = prior_sd,
                                                     log = TRUE) - prior_adj
             },
             lower = m_lower,
             upper = m_upper,
             method = "Brent",
             control = list(fnscale = -1))$value
  i <- integrate(f = function(x) {
    exp(log_likelihood(beta = x,
                       time = time,
                       event = event,
                       group = group) + dnorm(x = x,
                                              mean = prior_mean,
                                              sd = prior_sd,
                                              log = TRUE) - prior_adj - m)
  },
  lower = i_lower,
  upper = i_upper)$value
  r <- m + log(i)
  exp(r - log_lik)
}

#' Bayes factor for non-inferiority designs
#'
#' \code{\link{infer_bf}} computes a Bayes factor for non-inferiority designs
#' with a continuous dependent variable.
#'
#' The formulation of the null and alternative hypotheses for the
#' non-inferiority design differs depending on whether high or low scores on the
#' dependent variable represent non-inferiority. In the case where high scores
#' correspond to non-inferiority, the hypotheses are as follows: The null
#' hypothesis states that the population mean of the experimental group (e.g., a
#' new medication) is lower than the population mean of the control group (e.g.,
#' a placebo or an already existing medication) minus the non-inferiority
#' margin. The alternative hypothesis states that the population mean of the
#' experimental group is higher than the population mean of the control group
#' minus the non-inferiority margin. Thus, the null hypothesis goes in the
#' negative direction (i.e., H-) and the alternative hypothesis in the positive
#' direction (i.e., H+). In turn, in the case where low scores correspond to
#' non-inferiority, the hypotheses are as follows: The null hypothesis states
#' that the population mean of the experimental group is higher than the
#' population mean of the control group plus the non-inferiority margin. The
#' alternative hypothesis states that the population mean of the experimental
#' group is lower than the population mean of the control group plus the
#' non-inferiority margin. Thus, the null hypothesis goes in the positive
#' direction (i.e., H+) and the alternative hypothesis in the negative direction
#' (i.e., H-). The dependent variable must be continuous.
#'
#' Since the main goal of \code{\link{infer_bf}} is to establish
#' non-inferiority, the resulting Bayes factor quantifies evidence in favor of
#' the alternative hypothesis. In the case where high values represent
#' non-inferiority we have BF+- and in the case where low values represent
#' non-inferiority we have BF-+. Evidence for the null hypothesis can easily be
#' calculated by taking the reciprocal of the original Bayes factor (i.e., BF+-
#' = 1 / BF-+ and vice versa). Quantification of evidence in favor of the null
#' hypothesis is logically sound and legitimate within the Bayesian framework
#' (see e.g., van Ravenzwaaij et al., 2019).
#'
#' \code{\link{infer_bf}} can be utilized to calculate a Bayes factor based on
#' raw data (i.e., if arguments \code{x} and \code{y} are defined) or summary
#' statistics (i.e., if arguments \code{n_x}, \code{n_y}, \code{mean_x}, and
#' \code{mean_y} (or \code{ci_margin} and \code{ci_level} instead of \code{sd_x}
#' and \code{sd_y}) are defined). Arguments with 'x' as a name or suffix
#' correspond to the control group, whereas arguments with 'y' as a name or
#' suffix correspond to the experimental group.
#'
#' Since sometimes high scores on the dependent variable are considered
#' non-inferior (e.g., amount of social interactions) and sometimes rather the
#' low scores (e.g., severity of symptoms), the direction of non-inferiority can
#' be specified with the argument \code{direction}. For the case where high
#' values on the dependent variable indicate non-inferiority, 'high' (the
#' default) should be specified for the argument \code{direction}; if low values
#' on the dependent variable indicate non-inferiority, 'low' should be specified
#' for the argument \code{direction}.
#'
#' With the argument \code{ni_margin}, the non-inferiority margin can be
#' specified. \code{ni_margin} should be a positive number.' It can be declared
#' whether the non-inferiority margin is specified in standardized or
#' unstandardized units with the \code{ni_margin_std} argument, where TRUE,
#' corresponding to standardized units, is the default.
#'
#' For the calculation of the Bayes factor, a Cauchy prior density centered on 0
#' is chosen for the effect size under the alternative hypothesis. The standard
#' Cauchy distribution, with a location parameter of 0 and a scale parameter of
#' 1, resembles a standard Normal distribution, except that the Cauchy
#' distribution has less mass at the center but heavier tails (Liang et al.,
#' 2008; Rouder et al., 2009). The argument \code{prior_scale} specifies the
#' width of the Cauchy prior, which corresponds to half of the interquartile
#' range. Thus, by adjusting the Cauchy prior scale with \code{prior_scale},
#' different ranges of expected effect sizes can be emphasized. The default
#' prior scale is set to r = 1 / sqrt(2).
#'
#' \code{\link{infer_bf}} creates an S4 object of class
#' \linkS4class{baymedrNonInferiority}, which has multiple slots/entries (e.g.,
#' type of data, prior scale, Bayes factor, etc.; see Value). If it is desired
#' to store or extract solely the Bayes factor, the user can do this with
#' \code{\link{get_bf}}, by setting the S4 object as an argument (see Examples).
#'
#' @param ni_margin A numeric vector of length one, specifying the
#'   non-inferiority margin. The value should be a positive number.
#' @param ni_margin_std A logical vector of length one, specifying whether the
#'   non-inferiority margin (i.e., \code{ni_margin}) is given in standardized
#'   (TRUE; the default) or unstandardized (FALSE) units.
#' @param direction A character vector of length one, specifying the direction
#'   of non-inferior scores. 'low' indicates that low scores on the measure of
#'   interest correspond to a non-inferior outcome and 'high' (the default)
#'   indicates that high scores on the measure of interest correspond to a
#'   non-inferior outcome (see Details).
#' @inheritParams super_bf
#'
#' @return An S4 object of class \linkS4class{baymedrNonInferiority} is
#'   returned. Contained are a description of the model and the resulting Bayes
#'   factor: \itemize{ \item test: The type of analysis \item hypotheses: A
#'   statement of the hypotheses \itemize{ \item h0: The null hypothesis \item
#'   h1: The alternative hypothesis} \item ni_margin: The value for ni_margin in
#'   standardized and unstandardized units \itemize{ \item ni_mar_std: The
#'   standardized non-inferiority margin \item ni_mar_unstd: The unstandardized
#'   non-inferiority margin} \item data: A description of the data \itemize{
#'   \item type: The type of data ('raw' when arguments \code{x} and \code{y}
#'   are used or 'summary' when arguments \code{n_x}, \code{n_y}, \code{mean_x},
#'   \code{mean_y}, \code{sd_x}, and \code{sd_y} (or \code{ci_margin} and
#'   \code{ci_level} instead of \code{sd_x} and \code{sd_y}) are used) \item
#'   ...: values for the arguments used, depending on 'raw' or summary'} \item
#'   prior_scale: The width of the Cauchy prior distribution \item bf: The
#'   resulting Bayes factor } A summary of the model is shown by printing the
#'   object.
#'
#' @export
#' @import stats stringr
#'
#' @references
#'   Gronau, Q. F., Ly, A., & Wagenmakers, E.-J. (2020). Informed Bayesian
#'   t-tests. \emph{The American Statistician}, \emph{74}, 137-143.
#'
#'   Liang, F., Paulo, R., Molina, G., Clyde, M. A., & Berger, J. O. (2008).
#'   Mixtures of g priors for Bayesian variable selection. \emph{Journal of the
#'   American Statistical Association}, \emph{103}, 410-423.
#'
#'   Rouder, J. N., Speckman, P. L., Sun, D., Morey, R. D., & Iverson, G.
#'   (2009). Bayesian t tests for accepting and rejecting the null hypothesis.
#'   \emph{Psychonomic Bulletin & Review}, \emph{16}, 225-237.
#'
#'   van Ravenzwaaij, D., Monden, R., Tendeiro, J. N., & Ioannidis, J. P. A.
#'   (2019). Bayes factors for superiority, non-inferiority, and equivalence
#'   designs. \emph{BMC Medical Research Methodology}, \emph{19}, 71.
#'
#' @examples
#' ## infer_bf using raw data:
#'
#' # Assign model to variable.
#' infer_raw <- infer_bf(x = rnorm(100, 10, 15),
#'                       y = rnorm(130, 13, 10),
#'                       ni_margin = 1.5,
#'                       ni_margin_std = FALSE)
#'
#' # Extract Bayes factor from model.
#' get_bf(infer_raw)
#'
#' # ----------
#' # ----------
#'
#' ## infer_bf using summary statistics with data from Andersson et al. (2013).
#' ## Test at timepoint 1:
#'
#' # Assign model to variable.
#' infer_sum_t1 <- infer_bf(n_x = 33,
#'                          n_y = 32,
#'                          mean_x = 17.1,
#'                          mean_y = 13.6,
#'                          sd_x = 8,
#'                          sd_y = 9.8,
#'                          ni_margin = 2,
#'                          ni_margin_std = FALSE,
#'                          direction = "low")
#'
#' # Extract Bayes factor from model
#' get_bf(infer_sum_t1)
#'
#' # ----------
#' # ----------
#'
#' ## infer_bf using summary statistics with data from Andersson et al. (2013).
#' ## Test at timepoint 2:
#'
#' # Assign model to variable.
#' infer_sum_t2 <- infer_bf(n_x = 30,
#'                          n_y = 32,
#'                          mean_x = 13.5,
#'                          mean_y = 9.2,
#'                          sd_x = 8.7,
#'                          sd_y = 7.6,
#'                          ni_margin = 2,
#'                          ni_margin_std = FALSE,
#'                          direction = "low")
#'
#' # Extract Bayes factor from model
#' get_bf(infer_sum_t2)
infer_bf <- function(x = NULL,
                     y = NULL,
                     n_x = NULL,
                     n_y = NULL,
                     mean_x = NULL,
                     mean_y = NULL,
                     sd_x = NULL,
                     sd_y = NULL,
                     ci_margin = NULL,
                     ci_level = NULL,
                     ni_margin = NULL,
                     ni_margin_std = TRUE,
                     prior_scale = 1 / sqrt(2),
                     direction = "high") {
  if (any(!is.null(x),
          !is.null(y)) && any(!is.null(n_x),
                              !is.null(n_y),
                              !is.null(mean_x),
                              !is.null(mean_y),
                              !is.null(sd_x),
                              !is.null(sd_y),
                              !is.null(ci_margin),
                              !is.null(ci_level))) {
    stop(str_c(
      "Only 'x', 'y', and 'ni_margin' OR 'n_x', 'n_y', 'mean_x', 'mean_y', ",
      "'sd_x', 'sd_y', and 'ni_margin' (or 'ci_margin' and 'ci_level' instead ",
      "of 'sd_x' and 'sd_y') must be defined."
    ),
    call. = FALSE)
  }
  if (any(!is.null(x),
          !is.null(y))) {
    if (any(is.null(x),
            is.null(y),
            is.null(ni_margin))) {
      stop("All 'x', 'y', and 'ni_margin' must be defined.",
           call. = FALSE)
    }
  }
  if (any(!is.null(n_x),
          !is.null(n_y),
          !is.null(mean_x),
          !is.null(mean_y))) {
    if (any(is.null(n_x),
            is.null(n_y),
            is.null(mean_x),
            is.null(mean_y),
            is.null(ni_margin)) ||
        ((is.null(sd_x) || is.null(sd_y)) &&
         (is.null(ci_margin) || is.null(ci_level)))) {
      stop(str_c(
        "All 'n_x', 'n_y', 'mean_x', 'mean_y', 'sd_x', 'sd_y', and ",
        "'ni_margin' (or 'ci_margin' and 'ci_level' instead of 'sd_x' and ",
        "'sd_y') must be defined."
      ),
      call. = FALSE)
    }
    if (!xor(!is.null(sd_x) && !is.null(sd_y),
             !is.null(ci_margin) && !is.null(ci_level))) {
      stop(
        "Only 'sd_x' and 'sd_y' OR 'ci_margin' and 'ci_level' must be defined.",
        call. = FALSE
      )
    }
  }
  if (all(!is.null(n_x),
          !is.null(n_y),
          !is.null(mean_x),
          !is.null(mean_y)) && (xor(
            !is.null(sd_x) && !is.null(sd_y),
            !is.null(ci_margin) && !is.null(ci_level)
          ))) {
    if (!is.null(ci_level) && (length(ci_level) > 1 || ci_level <= 0 ||
                               ci_level >= 1 || !is.numeric(ci_level))) {
      stop("'ci_level' must be a single numeric value between 0 and 1.",
           call. = FALSE)
    }
    data <- list(type = "summary data",
                 data = list(n_x = n_x,
                             n_y = n_y,
                             mean_x = mean_x,
                             mean_y = mean_y,
                             sd_x = sd_x,
                             sd_y = sd_y,
                             ci_margin = ci_margin,
                             ci_level = ci_level))
  }
  if (!is.null(x) && !is.null(y)) {
    if (any(is.na(x)) || any(is.na(y))) {
      stop("'x' and 'y' must not contain missing values.",
           call. = FALSE)
    }
    if (any(is.infinite(x)) || any(is.infinite(y))) {
      stop("'x' and 'y' must not contain infinite values.",
           call. = FALSE)
    }
    if (!is.numeric(x) || !is.numeric(y)) {
      stop("'x' and 'y' must be numeric vectors.",
           call. = FALSE)
    }
    n_x <- length(x)
    n_y <- length(y)
    mean_x <- mean(x)
    mean_y <- mean(y)
    sd_x <- sd(x)
    sd_y <- sd(y)
    data <- list(type = "raw data",
                 data = list(x = x,
                             y = y))
  }
  if (!is.numeric(prior_scale) || length(prior_scale) > 1) {
    stop("'prior_scale' must be a single numeric value.",
         call. = FALSE)
  }
  if (!is.numeric(ni_margin) || length(ni_margin) > 1 || ni_margin < 0) {
    stop("'ni_margin' must be a single positive numeric value.",
         call. = FALSE)
  }
  if (!is.logical(ni_margin_std) || length(ni_margin_std) > 1) {
    stop("'ni_margin_std' must be a single locial value.",
         call. = FALSE)
  }
  if (!is.character(direction) || length(direction) > 1) {
    stop("'direction' must be a single character value.",
         call. = FALSE)
  }
  if (!is.null(sd_x) && !is.null(sd_y)) {
    sd_pooled <- sqrt(((n_x - 1) * sd_x ^ 2 + (n_y - 1) * sd_y ^ 2) /
                        (n_x + n_y - 2))
    se <- sd_pooled * sqrt(1 / n_x + 1 / n_y)
  } else {
    perc <- 1 - ((1 - ci_level) / 2)
    se <- ci_margin / qt(p = perc,
                         df = n_x + n_y - 2)
    sd_pooled <- se / sqrt(1 / n_x + 1 / n_y)
  }
  if (str_detect(direction,
                 "low")) {
    if (isFALSE(ni_margin_std)) {
      ni_mar_std <- ni_margin / sd_pooled
      ni_mar_unstd <- ni_margin
    } else {
      ni_mar_std <- ni_margin
      ni_mar_unstd <- ni_margin * sd_pooled
    }
    t_stat <- (mean_y - mean_x - ni_mar_unstd) / se
    res <- bf10_t(t = t_stat,
                  n1 = n_x,
                  n2 = n_y,
                  ind_samples = TRUE,
                  prior_loc = -ni_mar_std,
                  prior_scale = prior_scale,
                  prior_df = 1)
    bf <- res$bf_min0 / res$bf_plus0
    h0 <- "mu_y - mu_x > ni_margin"
    h1 <- "mu_y - mu_x < ni_margin"
  } else if (str_detect(direction,
                        "high")) {
    if (isFALSE(ni_margin_std)) {
      ni_mar_std <- -ni_margin / sd_pooled
      ni_mar_unstd <- -ni_margin
    } else {
      ni_mar_std <- -ni_margin
      ni_mar_unstd <- -ni_margin * sd_pooled
    }
    t_stat <- (mean_y - mean_x - ni_mar_unstd) / se
    res <- bf10_t(t = t_stat,
                  n1 = n_x,
                  n2 = n_y,
                  ind_samples = TRUE,
                  prior_loc = -ni_mar_std,
                  prior_scale = prior_scale,
                  prior_df = 1)
    bf <- res$bf_plus0 / res$bf_min0
    h0 <- "mu_y - mu_x < -ni_margin"
    h1 <- "mu_y - mu_x > -ni_margin"
  } else {
    stop("'direction' must be one of 'low' or 'high'.",
         call. = FALSE)
  }
  test <- "Non-inferiority analysis"
  hypotheses <- list(h0 = h0,
                     h1 = h1)
  ni_margin <- list(ni_mar_std = abs(ni_mar_std),
                    ni_mar_unstd = abs(ni_mar_unstd))
  new(Class = "baymedrNonInferiority",
      test = test,
      hypotheses = hypotheses,
      ni_margin = ni_margin,
      data = data,
      prior_scale = prior_scale,
      bf = bf)
}

context("test-infer_bf")

con <- rnorm(100)
exp <- rnorm(100)

test_that("infer_bf yields correct S4 class", {
  expect_is(
    infer_bf(x = con,
             y = exp,
             ni_margin = 0.5),
    "baymedrNonInferiority"
  )
  expect_is(
    infer_bf(n_x = 100,
             n_y = 100,
             mean_x = 10,
             mean_y = 10,
             sd_x = 5,
             sd_y = 5,
             ni_margin = 0.5),
    "baymedrNonInferiority"
  )
})

test_that("infer_bf yields numeric Bayes factor", {
  expect_true(
    is.numeric(infer_bf(x = con,
                        y = exp,
                        ni_margin = 0.5)@bf)
  )
  expect_true(
    is.numeric(infer_bf(n_x = 100,
                        n_y = 100,
                        mean_x = 10,
                        mean_y = 10,
                        sd_x = 5,
                        sd_y = 5,
                        ni_margin = 0.5)@bf)
  )
})

test_that("infer_bf gives correct error messages", {
  expect_error(
    infer_bf(x = con,
             y = exp,
             n_x = 80,
             ni_margin = 0.5),
    str_c(
      "Only 'x', 'y', and 'ni_margin' OR 'n_x', 'n_y', 'mean_x', 'mean_y', ",
      "'sd_x', 'sd_y', and 'ni_margin' (or 'ci_margin' and 'ci_level' instead ",
      "of 'sd_x' and 'sd_y') must be defined."
    ),
    fixed = TRUE
  )
  expect_error(
    infer_bf(x = con),
    "All 'x', 'y', and 'ni_margin' must be defined.",
    fixed = TRUE
  )
  expect_error(
    infer_bf(n_x = 100,
             n_y = 100,
             mean_x = 10,
             sd_x = 5),
    str_c(
      "All 'n_x', 'n_y', 'mean_x', 'mean_y', 'sd_x', 'sd_y', and ",
      "'ni_margin' (or 'ci_margin' and 'ci_level' instead of 'sd_x' and ",
      "'sd_y') must be defined."
    ),
    fixed = TRUE
  )
  expect_error(
    infer_bf(x = con,
             y = c(exp[1:99], NA),
             ni_margin = -0.5),
    "'x' and 'y' must not contain missing values.",
    fixed = TRUE
  )
  expect_error(
    infer_bf(x = con,
             y = c(exp[1:99], Inf),
             ni_margin = -0.5),
    "'x' and 'y' must not contain infinite values.",
    fixed = TRUE
  )
  expect_error(
    infer_bf(x = con,
             y = rep(letters[1:10],
                     each = 10),
             ni_margin = -0.5),
    "'x' and 'y' must be numeric vectors.",
    fixed = TRUE
  )
  expect_error(
    infer_bf(x = con,
             y = exp,
             ni_margin = -0.5,
             prior_scale = "x"),
    "'prior_scale' must be a single numeric value.",
    fixed = TRUE
  )
  expect_error(
    infer_bf(x = con,
             y = exp,
             ni_margin = -0.5,
             prior_scale = c(1, 2)),
    "'prior_scale' must be a single numeric value.",
    fixed = TRUE
  )
  expect_error(
    infer_bf(x = con,
             y = exp,
             ni_margin = "0.5"),
    "'ni_margin' must be a single positive numeric value.",
    fixed = TRUE
  )
  expect_error(
    infer_bf(x = con,
             y = exp,
             ni_margin = c(-0.5, 0.5)),
    "'ni_margin' must be a single positive numeric value.",
    fixed = TRUE
  )
  expect_error(
    infer_bf(x = con,
             y = exp,
             ni_margin = -0.5),
    "'ni_margin' must be a single positive numeric value.",
    fixed = TRUE
  )
  expect_error(
    infer_bf(x = con,
             y = exp,
             ni_margin = 0.5,
             direction = 5),
    "'direction' must be a single character value.",
    fixed = TRUE
  )
  expect_error(
    infer_bf(x = con,
             y = exp,
             ni_margin = 0.5,
             direction = c("low", "high")),
    "'direction' must be a single character value.",
    fixed = TRUE
  )
  expect_error(
    infer_bf(x = con,
             y = exp,
             ni_margin = 0.5,
             direction = "abc"),
    "'direction' must be one of 'low' or 'high'.",
    fixed = TRUE
  )
  expect_error(
    infer_bf(n_x = 100,
             n_y = 100,
             mean_x = 0,
             mean_y = 2,
             ci_margin = 5,
             ci_level = 1.5,
             ni_margin = 0.5),
    "'ci_level' must be a single numeric value between 0 and 1.",
    fixed = TRUE
  )
  expect_error(
    infer_bf(n_x = 100,
             n_y = 100,
             mean_x = 0,
             mean_y = 2,
             ci_margin = 5,
             ci_level = "high",
             ni_margin = 0.5),
    "'ci_level' must be a single numeric value between 0 and 1.",
    fixed = TRUE
  )
  expect_error(
    infer_bf(n_x = 100,
             n_y = 100,
             mean_x = 0,
             mean_y = 2,
             ci_margin = 5,
             ci_level = c(0.3, 0.7),
             ni_margin = 0.5),
    "'ci_level' must be a single numeric value between 0 and 1.",
    fixed = TRUE
  )
})

#' Extract Bayes factor from S4 object
#'
#' \code{\link{get_bf}} extracts the Bayes factor from an S4 object (i.e.,
#' \linkS4class{baymedrSuperiority}, \linkS4class{baymedrEquivalence},
#' \linkS4class{baymedrNonInferiority}),
#' \linkS4class{baymedrCoxProportionalHazards}, and
#' \linkS4class{baymedrCoxProportionalHazardsMulti}.
#'
#' @param object An S4 object of class \linkS4class{baymedrSuperiority},
#'   \linkS4class{baymedrEquivalence}, \linkS4class{baymedrNonInferiority},
#'   \linkS4class{baymedrCoxProportionalHazards}, or
#'   \linkS4class{baymedrCoxProportionalHazardsMulti}.
#'
#' @return A numeric vector, providing the Bayes factor(s) from an S4 object.
#'
#' @import survival
#' @export
#'
#' @examples
#' # Extract Bayes factor from a baymedrSuperiority object using raw data:
#' mod_super <- super_bf(x = rnorm(100, 10, 15),
#'                       y = rnorm(130, 13, 10))
#'
#' get_bf(object = mod_super)
#'
#' # Extract Bayes factor from a baymedrEquivalence object using raw data:
#' mod_equiv <- equiv_bf(x = rnorm(100, 10, 15),
#'                       y = rnorm(130, 13, 10))
#'
#' get_bf(object = mod_equiv)
#'
#' # Extract Bayes factor from a baymedrNonInferiority object using raw data:
#' mod_infer <- infer_bf(x = rnorm(100, 10, 15),
#'                       y = rnorm(130, 13, 10),
#'                       ni_margin = 1)
#'
#' get_bf(object = mod_infer)
#'
#' # Extract Bayes factor from a baymedrCoxProportionalHazards object:
#' data <- survival::aml
#' names(data) <- c("time", "event", "group")
#' data$group <- ifelse(test = data$group == "Maintained",
#'                      yes = 0,
#'                      no = 1)
#'
#' mod_coxph <- coxph_bf(data = data)
#'
#' get_bf(object = mod_coxph)
get_bf <- function(object) {
  if (all(!sapply(c("baymedrSuperiority",
                    "baymedrEquivalence",
                    "baymedrNonInferiority",
                    "baymedrCoxProportionalHazards",
                    "baymedrCoxProportionalHazardsMulti"),
                  function(x) {
                    is(object = object,
                       class2 = x)
                  }))) {
    stop(str_c(
      "Bayes factors can only be extracted from S4 objects of classes ",
      "'baymedrEquivalence', 'baymedrNonInferiority', ",
      "'baymedrSuperiority', 'baymedrCoxProportionalHazards', and ",
      "'baymedrCoxProportionalHazardsMulti'."
    ),
    call. = FALSE)
  }
  object@bf
}

#' Simulate data for Cox proportional hazards regression
#'
#' \code{\link{coxph_data_sim}} simulates data for Cox proportional hazards
#' regression models with one dichotomous independent variable based on summary
#' statistics.
#'
#' Particle swarm optimization, as implemented by \code{\link[pso]{psoptim}} is
#' used to simulate one or multiple datasets that match certain summary
#' statistics. The algorithm uses as many parameters as there cases in the
#' dataset that is to be simulated. Therefore, using
#' \code{\link{coxph_data_sim}} becomes more time-consuming the larger the
#' sample size.
#'
#' The relevant summary statistics that are used in the optimization process
#' are: \itemize{ \item cox_hr \itemize{ \item Hazard ratio
#' between the experimental and control conditions based on a Cox proportional
#' hazards regression model. \item Lower boundary of the x% confidence interval
#' of the hazard ratio between the experimental and control conditions based on
#' a Cox proportional hazards regression model. \item Upper boundary of the x%
#' confidence interval of the hazard ratio between the experimental and control
#' conditions based on a Cox proportional hazards regression model. } }
#'
#' \code{\link{coxph_data_sim}} creates a list with as many elements as
#' specified by the argument \code{n_data}. Each element consists of a list that
#' entails the resulting simulated data and the optimization results of the data
#' simulation process.
#'
#' @param n_data The number of datasets to be simulated. The default is 1.
#' @param ns_c Sample size of the control condition.
#' @param ns_e Sample size of the experimental condition.
#' @param ne_c Number of events (e.g., death) in the control condition.
#' @param ne_e Number of events (e.g., death) in the experimental condition.
#' @param cox_hr A numeric vector of length 3, indicating the hazard ratio
#'   between the experimental and control conditions based on a Cox proportional
#'   hazards regression model, the lower boundary of the x% confidence interval
#'   of the hazard ratio between the experimental and control conditions based
#'   on a Cox proportional hazards regression model, and the upper boundary of
#'   the x% confidence interval of the hazard ratio between the experimental and
#'   control conditions based on a Cox proportional hazards regression model,
#'   respectively. The hazard ratio must be provided. The confidence interval
#'   boundaries are optional; if missing they should be given as NA.
#' @param cox_hr_ci_level Confidence level of the x% confidence interval of the
#'   hazard ratio between the experimental and control conditions based on a Cox
#'   proportional hazards regression model. The default is 0.95.
#' @param max_t The maximum allowed survival/censoring time. The default is 100.
#' @param cores The number of cores to be used in the data simulation process.
#'   The default is 1. Note that it is only useful to use more than 1 core if
#'   more than 1 dataset is simulated; ideally, \code{n_data} should be a
#'   multiple of \code{cores}.
#' @param ... Arguments passed to the \code{control} argument of
#'   \code{\link[pso]{psoptim}} (e.g. maxit, maxit.stagnate). Be aware that
#'   \code{\link{coxph_data_sim}} uses default values that are not the
#'   default in \code{\link[pso]{psoptim}}. Specifically,
#'   \code{\link{coxph_data_sim}} uses \code{maxit = 5000}, and
#'   \code{maxit.stagnate = ceiling(maxit / 5)}.
#'
#' @return A list of length \code{n_data} is returned. Each element of that list
#'   contains one simulated dataset and information about the optimization
#'   process: \itemize{ \item data: A data.frame containing the following
#'   columns: \itemize{ \item time: Survival/censoring times. \item event:
#'   Indication of whether an event happened (1) or not (0). \item group:
#'   Indication of whether case belongs to control condition (0) or experimental
#'   condition (1). } \item optim: Results of particle swarm optimization. See
#'   the Value section in \code{\link[pso]{psoptim}} }.
#'
#' @seealso \code{\link{coxph_bf}} and \code{\link[pso]{psoptim}}.
#'
#' @export
#' @import doParallel foreach parallel pso stringr survival
#'
#' @references
#'   Harrell, F. R. (2015). Regression modeling strategies: Withapplications to
#'   linear models, logistic regression, and survival analysis (2nd ed.).
#'   Springer.
#'
#'   Kennedy, J., & Eberhart, R. (1995). Particle swarm optimization.
#'   \emph{Proceedings of ICNN'95 - International Conference on Neural
#'   Networks}, \emph{4}, 1942-1948.
#'
#'   Shi, Y., & Eberhart, R. (1998). A modified particle swarm optimizer.
#'   \emph{1998 IEEE International Conference on Evolutionary Computation
#'   Proceedings. IEEE World Congress on Computational Intelligence}, 69-73.
#'
#' @examples
#' # Pretend we extracted the following summary statistics from an article.
#' ns_c <- 20
#' ns_e <- 56
#' ne_c <- 18
#' ne_e <- 40
#' cox_hr <- c(0.433, 0.242, 0.774)
#' cox_hr_ci_level <- 0.95
#'
#' # We want to simulate 3 datasets. We do not need a very precise match of the
#' # summary statistics to the real summary statistics. Therefore, for
#' # demonstration purposes we only use 1/200 of the default number of
#' # optimization iterations (i.e., (1 / 200) * 5000).
#' sim_data <- coxph_data_sim(n_data = 3,
#'                            ns_c = ns_c,
#'                            ns_e = ns_e,
#'                            ne_c = ne_c,
#'                            ne_e = ne_e,
#'                            cox_hr = cox_hr,
#'                            cox_hr_ci_level = cox_hr_ci_level,
#'                            max_t = 100,
#'                            maxit = 25)
coxph_data_sim <- function(n_data = 1,
                           ns_c,
                           ns_e,
                           ne_c,
                           ne_e,
                           cox_hr,
                           cox_hr_ci_level = 0.95,
                           max_t = 100,
                           cores = 1,
                           ...) {
  if (length(n_data) != 1 || n_data < 1) {
    stop("'n_data' must be a single positive integer.",
         call. = FALSE)
  }
  if (length(ns_c) != 1 || ns_c < 1 || length(ns_e) != 1 || ns_e < 1 ||
      length(ne_c) != 1 || ne_c < 1 || length(ne_e) != 1 || ne_e < 1) {
    stop("'ns_c', 'ns_e', 'ne_c', and 'ne_e' must be single positive integers.",
         call. = FALSE)
  }
  if (length(cox_hr) != 3 || !is.numeric(cox_hr) || is.na(cox_hr[1]) ||
      !all(cox_hr > 0, na.rm = TRUE) ||
      ifelse(test = !is.na(cox_hr[2]),
             yes = cox_hr[2] >= cox_hr[1],
             no = FALSE) ||
      ifelse(test = !is.na(cox_hr[3]),
             yes = cox_hr[3] <= cox_hr[1],
             no = FALSE)) {
    stop(str_c("'cox_hr' must be a numeric vector of length 3 containing only ",
               "positive values. The second entry must be lower than the first entry and ",
               "the third entry must be higher than the first entry."),
         call. = FALSE)
  }
  if (cox_hr_ci_level <= 0 || cox_hr_ci_level >= 1) {
    stop("'cox_hr_ci_level' must be between 0 and 1.",
         call. = FALSE)
  }
  if (length(max_t) != 1 || !is.numeric(max_t) || max_t <= 0) {
    stop("'max_t' must be a single positive number.",
         call. = FALSE)
  }
  if (length(cores) != 1 || cores < 1) {
    stop("'cores' must be a single positive integer.",
         call. = FALSE)
  }
  pso_args <- as.list(match.call(expand.dots = FALSE)$...)
  pso_args <- lapply(X = pso_args,
                     FUN = eval)
  if (is.null(pso_args$maxit)) {
    pso_args$maxit <- 300
  }
  if (is.null(pso_args$maxit.stagnate)) {
    pso_args$maxit.stagnate <- ceiling(pso_args$maxit / 5)
  }
  if (cores == 1) {
    message(str_c("Running in serial.\n",
                  "Computations might take some time."))
    res <-
      foreach(
        i = 1:n_data,
        .packages = c("pso", "survival")
      ) %do% {
        time <- runif(n = ns_c + ns_e,
                      min = 1,
                      max = max_t)
        event <- c(rep(c(0, 1), c(ns_c - ne_c, ne_c)),
                   rep(c(0, 1), c(ns_e - ne_e, ne_e)))
        group <- rep(c(0, 1), c(ns_c, ns_e))
        c <- do.call(what = psoptim,
                     args = list(par = rep(NA, length(time)),
                                 fn = loss,
                                 cox_hr = cox_hr,
                                 time = time,
                                 event = event,
                                 group = group,
                                 cox_hr_ci_level = cox_hr_ci_level,
                                 lower = log(1 / time),
                                 upper = log(max_t / time),
                                 control = pso_args))
        time <- time * exp(c$par)
        list(data = data.frame(time = time,
                               event = event,
                               group = group),
             optim = c)
      }
    nc <- nchar(n_data)
    names(res) <- paste0("data_", formatC(x = 1:n_data,
                                          width = nc,
                                          flag = "0"))
    res
  } else {
    if (!getDoParRegistered()) {
      message(str_c("Running in parallel using new parallel backend.\n",
                    "Computations might take some time."))
      cl <- makeCluster(cores)
      registerDoParallel(cl)
      res <-
        foreach(
          i = 1:n_data,
          .packages = c("pso", "survival")
        ) %dopar% {
          time <- runif(n = ns_c + ns_e,
                        min = 1,
                        max = max_t)
          event <- c(rep(c(0, 1), c(ns_c - ne_c, ne_c)),
                     rep(c(0, 1), c(ns_e - ne_e, ne_e)))
          group <- rep(c(0, 1), c(ns_c, ns_e))
          c <- do.call(what = psoptim,
                       args = list(par = rep(NA, length(time)),
                                   fn = loss,
                                   cox_hr = cox_hr,
                                   time = time,
                                   event = event,
                                   group = group,
                                   cox_hr_ci_level = cox_hr_ci_level,
                                   lower = log(1 / time),
                                   upper = log(max_t / time),
                                   control = pso_args))
          time <- time * exp(c$par)
          list(data = data.frame(time = time,
                                 event = event,
                                 group = group),
               optim = c)
        }
      nc <- nchar(n_data)
      names(res) <- paste0("data_", formatC(x = 1:n_data,
                                            width = nc,
                                            flag = "0"))
      stopCluster(cl)
      # env <- foreach:::.foreachGlobals
      # rm(list = ls(name = env),
      #    pos = env)
      setDoSeq()
      res
    } else {
      message(str_c("Running in parallel using existing parallel backend.\n",
                    "Computations might take some time."))
      res <-
        foreach(
          i = 1:n_data,
          .packages = c("pso", "survival")
        ) %dopar% {
          time <- runif(n = ns_c + ns_e,
                        min = 1,
                        max = max_t)
          event <- c(rep(c(0, 1), c(ns_c - ne_c, ne_c)),
                     rep(c(0, 1), c(ns_e - ne_e, ne_e)))
          group <- rep(c(0, 1), c(ns_c, ns_e))
          c <- do.call(what = psoptim,
                       args = list(par = rep(NA, length(time)),
                                   fn = loss,
                                   cox_hr = cox_hr,
                                   time = time,
                                   event = event,
                                   group = group,
                                   cox_hr_ci_level = cox_hr_ci_level,
                                   lower = log(1 / time),
                                   upper = log(max_t / time),
                                   control = pso_args))
          time <- time * exp(c$par)
          list(data = data.frame(time = time,
                                 event = event,
                                 group = group),
               optim = c)
        }
      nc <- nchar(n_data)
      names(res) <- paste0("data_", formatC(x = 1:n_data,
                                            width = nc,
                                            flag = "0"))
      res
    }
  }
}

context("test-coxph_data_sim")

ns_c <- 20
ns_e <- 56
ne_c <- 18
ne_e <- 40
cox_hr <- c(0.433, 0.242, 0.774)
cox_hr_ci_level <- 0.95
test_that("coxph_bf yields correct type", {
  expect_is(
    coxph_data_sim(n_data = 2,
                   ns_c = ns_c,
                   ns_e = ns_e,
                   ne_c = ne_c,
                   ne_e = ne_e,
                   cox_hr = cox_hr,
                   cox_hr_ci_level = cox_hr_ci_level,
                   max_t = 100,
                   cores = 1,
                   maxit = 25),
    "list"
  )
})

test_that("coxph_data_sim gives correct error messages", {
  expect_error(
    coxph_data_sim(n_data = 0,
                   ns_c = ns_c,
                   ns_e = ns_e,
                   ne_c = ne_c,
                   ne_e = ne_e,
                   cox_hr = cox_hr,
                   cox_hr_ci_level = cox_hr_ci_level,
                   max_t = 100,
                   cores = 1,
                   maxit = 25),
    "'n_data' must be a single positive integer.",
    fixed = TRUE
  )
  expect_error(
    coxph_data_sim(n_data = 2,
                   ns_c = 0,
                   ns_e = ns_e,
                   ne_c = ne_c,
                   ne_e = ne_e,
                   cox_hr = cox_hr,
                   cox_hr_ci_level = cox_hr_ci_level,
                   max_t = 100,
                   cores = 1,
                   maxit = 25),
    "'ns_c', 'ns_e', 'ne_c', and 'ne_e' must be single positive integers.",
    fixed = TRUE
  )
  expect_error(
    coxph_data_sim(n_data = 2,
                   ns_c = ns_c,
                   ns_e = ns_e,
                   ne_c = ne_c,
                   ne_e = ne_e,
                   cox_hr = c(NA, 4, 6),
                   cox_hr_ci_level = cox_hr_ci_level,
                   max_t = 100,
                   cores = 1,
                   maxit = 25),
    str_c("'cox_hr' must be a numeric vector of length 3 containing only ",
          "positive values. The second entry must be lower than the first ",
          "entry and the third entry must be higher than the first entry."),
    fixed = TRUE
  )
  expect_error(
    coxph_data_sim(n_data = 2,
                   ns_c = ns_c,
                   ns_e = ns_e,
                   ne_c = ne_c,
                   ne_e = ne_e,
                   cox_hr = cox_hr,
                   cox_hr_ci_level = 95,
                   max_t = 100,
                   cores = 1,
                   maxit = 25),
    "'cox_hr_ci_level' must be between 0 and 1.",
    fixed = TRUE
  )
  expect_error(
    coxph_data_sim(n_data = 2,
                   ns_c = ns_c,
                   ns_e = ns_e,
                   ne_c = ne_c,
                   ne_e = ne_e,
                   cox_hr = cox_hr,
                   cox_hr_ci_level = cox_hr_ci_level,
                   max_t = c(5, 3),
                   cores = 1,
                   maxit = 25),
    "'max_t' must be a single positive number.",
    fixed = TRUE
  )
  expect_error(
    coxph_data_sim(n_data = 2,
                   ns_c = ns_c,
                   ns_e = ns_e,
                   ne_c = ne_c,
                   ne_e = ne_e,
                   cox_hr = cox_hr,
                   cox_hr_ci_level = cox_hr_ci_level,
                   max_t = 100,
                   cores = 0,
                   maxit = 25),
    "'cores' must be a single positive integer.",
    fixed = TRUE
  )
})

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/def_classes.R
\docType{class}
\name{model-classes}
\alias{model-classes}
\alias{baymedrSuperiority-class}
\alias{baymedrEquivalence-class}
\alias{baymedrNonInferiority-class}
\alias{baymedrCoxProportionalHazards-class}
\alias{baymedrCoxProportionalHazardsMulti-class}
\title{S4 classes to represent different models}
\description{
The S4 classes \linkS4class{baymedrSuperiority},
\linkS4class{baymedrEquivalence}, \linkS4class{baymedrNonInferiority},
\linkS4class{baymedrCoxProportionalHazards}, and
\linkS4class{baymedrCoxProportionalHazardsMulti} represent models for the
superiority (\code{\link{super_bf}}), equivalence (\code{\link{equiv_bf}}),
non-inferiority (\code{\link{infer_bf}}), and Cox proportional hazards
(\code{\link{coxph_bf}}) models, respectively.
}
\section{Slots}{

\describe{
\item{\code{test}}{Type of test that was conducted.}

\item{\code{hypotheses}}{The hypotheses that are tested.}

\item{\code{data}}{The type of data that was used.}

\item{\code{prior_scale}}{The Cauchy prior scale that was used.}

\item{\code{bf}}{The resulting Bayes factor.}

\item{\code{interval}}{The equivalence interval in case of \code{\link{equiv_bf}}.}

\item{\code{ni_margin}}{The non-inferiority margin in case of \code{\link{infer_bf}}.}

\item{\code{prior}}{The mean and standard deviation of the Normal prior in case of
\code{\link{coxph_bf}}.}
}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/coxph_bf.R
\name{coxph_bf}
\alias{coxph_bf}
\title{Bayes factor for Cox proportional hazards regression}
\usage{
coxph_bf(
  data,
  null_value = 0,
  alternative = "two.sided",
  direction = NULL,
  prior_mean = 0,
  prior_sd = 1
)
}
\arguments{
\item{data}{A data.frame or a list resulting from calling
\code{coxph_data_sim}. In the former case, the first column represents the
survival/censoring times, the second column indicates whether an event
(e.g., death) has happened (1) or not (0), and the third column represents
the dummy-coded independent variable, where the control group is coded with
0 and the experimental group with 1.}

\item{null_value}{The value of the point null hypothesis for the beta
coefficient. The default is a null value of 0.}

\item{alternative}{A string specifying whether the alternative hypothesis is
two-sided ("two.sided"; the default) or one-sided ("one.sided").}

\item{direction}{A string specifying the direction of the one-sided
alternative hypothesis. This is ignored if
\code{alternative = "two.sided"}. Possible options are "low" and "high".}

\item{prior_mean}{Mean of the Normal prior for the beta parameter. The
default is a mean of 0.}

\item{prior_sd}{Standard deviation of the Normal prior for the beta
parameter. The default is a standard deviation of 1.}
}
\value{
An S4 object of class \linkS4class{baymedrCoxProportionalHazards} is
  returned. Contained are a description of the model and the resulting Bayes
  factor: \itemize{ \item test: The type of analysis \item hypotheses: A
  statement of the hypotheses \itemize{ \item h0: The null hypothesis
  \item h1: The alternative hypothesis} \item prior: The parameters of the
  Normal prior on beta \item bf: The resulting Bayes factor } A summary of
  the model is shown by printing the object.
}
\description{
\code{\link{coxph_bf}} computes a Bayes factor for Cox proportional hazards
regression models with one dichotomous independent variable.
}
\details{
The Cox proportional hazards model has the following hypotheses: The null
hypothesis (i.e., H0) states that the population hazard ratio between the
experimental (e.g., a new medication) and the control group (e.g., a placebo
or an already existing medication) is equal to 1 (i.e., beta = 0). The
alternative hypothesis can be two-sided or one-sided (either negative or
positive).

Since the main goal of \code{\link{coxph_bf}} is to establish that the
hazard ratio is not equal to 1, the resulting Bayes factor quantifies
evidence in favor of the alternative hypothesis. For a two-sided alternative
hypothesis, we have BF10; for a negative one-sided alternative hypothesis, we
have BF-0; and for a positive one-sided alternative hypothesis, we have BF+0.
Evidence for the null hypothesis can easily be calculated by taking the
reciprocal of the original Bayes factor (i.e., BF01 = 1 / BF10).
Quantification of evidence in favor of the null hypothesis is logically sound
and legitimate within the Bayesian framework (see e.g., van Ravenzwaaij et
al., 2019).

For the calculation of the Bayes factor, a Normal prior density is chosen for
beta under the alternative hypothesis. The arguments \code{prior_mean} and
\code{prior_sd} specify the mean and standard deviation of the Normal prior,
respectively. By adjusting the Normal prior, different ranges of expected
effect sizes can be emphasized. The default is a Normal prior with a mean of
0 and a standard deviation of 1.

Note that at the moment the model specifications are limited. That is, it is
only possible to have a single dichotomous independent variable. Further, at
the moment only a Normal prior is supported. Lastly, only the Efron partial
likelihood and not the many other options are supported.

\code{\link{coxph_bf}} creates an S4 object of class
\linkS4class{baymedrCoxProportionalHazards}, which has multiple slots/entries
(e.g., prior, Bayes factor, etc.; see Value). If it is desired to store or
extract solely the Bayes factor, the user can do this with
\code{\link{get_bf}}, by setting the S4 object as an argument (see Examples).
}
\examples{
# Load aml dataset from the survival R package.
data <- survival::aml
data$x <- ifelse(test = data$x == "Maintained",
                 yes = 0,
                 no = 1)
names(data) <- c("time", "event", "group")

# Assign model to variable.
coxph_mod <- coxph_bf(data = data,
                      null_value = 0,
                      alternative = "one.sided",
                      direction = "high",
                      prior_mean = 0,
                      prior_sd = 1)

# Extract Bayes factor from variable.
get_bf(coxph_mod)
}
\references{
Harrell, F. R. (2015). Regression modeling strategies: With applications to
  linear models, logistic regression, and survival analysis (2nd ed.).
  Springer.

  van Ravenzwaaij, D., Monden, R., Tendeiro, J. N., & Ioannidis, J. P. A.
  (2019). Bayes factors for superiority, non-inferiority, and equivalence
  designs. \emph{BMC Medical Research Methodology}, \emph{19}, 71.
}
\seealso{
\code{\link{coxph_data_sim}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/equiv_bf.R
\name{equiv_bf}
\alias{equiv_bf}
\title{Bayes factor for equivalence designs}
\usage{
equiv_bf(
  x = NULL,
  y = NULL,
  n_x = NULL,
  n_y = NULL,
  mean_x = NULL,
  mean_y = NULL,
  sd_x = NULL,
  sd_y = NULL,
  ci_margin = NULL,
  ci_level = NULL,
  interval = 0,
  interval_std = TRUE,
  prior_scale = 1/sqrt(2)
)
}
\arguments{
\item{x}{A numeric vector of observations for the control group.}

\item{y}{A numeric vector of observations for the experimental group.}

\item{n_x}{A numeric vector of length one, specifying the sample size of the
control group.}

\item{n_y}{A numeric vector of length one, specifying the sample size of the
experimental group.}

\item{mean_x}{A numeric vector of length one, specifying the mean of the
dependent variable in the control group.}

\item{mean_y}{A numeric vector of length one, specifying the mean of the
dependent variable in the experimental group.}

\item{sd_x}{A numeric vector of length one, specifying the standard deviation
of the dependent variable in the control group. Only \code{sd_x} and
\code{sd_y} \strong{OR} \code{ci_margin} and \code{ci_level} should be
defined (see Details).}

\item{sd_y}{A numeric vector of length one, specifying the standard deviation
of the dependent variable in the experimental group. Only \code{sd_x} and
\code{sd_y} \strong{OR} \code{ci_margin} and \code{ci_level} should be
defined (see Details).}

\item{ci_margin}{A numeric vector of length one, specifying the margin of the
confidence interval (i.e., the width of the confidence interval divided by
2) of the mean difference on the dependent variable between the
experimental and control groups. The value should be a positive number Only
\code{sd_x} and \code{sd_y} \strong{OR} \code{ci_margin} and
\code{ci_level} should be defined (see Details).}

\item{ci_level}{A numeric vector of length one, specifying the confidence
level of \code{ci_margin}. The value must be between 0 and 1 (e.g., 0.95
for a 95\% confidence interval). Only \code{sd_x} and \code{sd_y}
\strong{OR} \code{ci_margin} and \code{ci_level} should be defined (see
Details).}

\item{interval}{A numeric vector of length one or two, specifying the
boundaries of the equivalence interval. If a numeric vector of length one
is specified, a symmetric equivalence interval will be used (e.g., a 0.1 is
equivalent to c(-0.1, 0.1)). A numeric vector of length two provides the
possibility to specify an asymmetric equivalence interval (e.g., c(-0.1,
0.2)). The default is 0, indicating a point null hypothesis rather than an
interval (see Details).}

\item{interval_std}{A logical vector of length one, specifying whether the
equivalence interval (i.e., \code{interval}) is given in standardized
(TRUE; the default) or unstandardized (FALSE) units.}

\item{prior_scale}{A numeric vector of length one, specifying the scale of
the Cauchy prior distribution for the effect size under the alternative
hypothesis (see Details). The default value is r = 1 / sqrt(2).}
}
\value{
An S4 object of class \linkS4class{baymedrEquivalence} is returned.
  Contained are a description of the model and the resulting Bayes factor:
  \itemize{ \item test: The type of analysis \item hypotheses: A statement of
  the hypotheses \itemize{ \item h0: The null hypothesis \item h1: The
  alternative hypothesis} \item interval: Specification of the equivalence
  interval in standardized and unstandardized units \itemize{ \item
  lower_std: The standardized lower boundary of the equivalence interval
  \item upper_std: The standardized upper boundary of the equivalence
  interval \item lower_unstd: The unstandardized lower boundary of the
  equivalence interval \item upper_unstd: The unstandardized upper boundary
  of the equivalence interval} \item data: A description of the data
  \itemize{ \item type: The type of data ('raw' when arguments \code{x} and
  \code{y} are used or 'summary' when arguments \code{n_x}, \code{n_y},
  \code{mean_x}, \code{mean_y}, \code{sd_x}, and \code{sd_y} (or
  \code{ci_margin} and \code{ci_level} instead of \code{sd_x} and
  \code{sd_y}) are used) \item ...: values for the arguments used, depending
  on 'raw' or summary'} \item prior_scale: The width of the Cauchy prior
  distribution \item bf: The resulting Bayes factor } A summary of the model
  is shown by printing the object.
}
\description{
\code{\link{equiv_bf}} computes a Bayes factor for equivalence designs with a
continuous dependent variable.
}
\details{
The equivalence design has the following hypotheses: The null hypothesis
(i.e., H0) states that the population means of the experimental group (e.g.,
a new medication) and the control group (e.g., a placebo or an already
existing medication) are (practically) equivalent; the alternative hypothesis
(i.e., H1) states that the population means of the two groups are not
equivalent. The dependent variable must be continuous.

Since the main goal of \code{\link{equiv_bf}} is to establish equivalence,
the resulting Bayes factor quantifies evidence in favor of the null
hypothesis (i.e., BF01). Evidence for the alternative hypothesis can easily
be calculated by taking the reciprocal of the original Bayes factor (i.e.,
BF10 = 1 / BF01). Quantification of evidence in favor of the null hypothesis
is logically sound and legitimate within the Bayesian framework (see e.g.,
van Ravenzwaaij et al., 2019).

\code{\link{equiv_bf}} can be utilized to calculate a Bayes factor based on
raw data (i.e., if arguments \code{x} and \code{y} are defined) or summary
statistics (i.e., if arguments \code{n_x}, \code{n_y}, \code{mean_x}, and
\code{mean_y} are defined). In the latter case, either values for the
arguments \code{sd_x} and \code{sd_y} \strong{OR} \code{ci_margin} and
\code{ci_level} can be supplied. Arguments with 'x' as a name or suffix
correspond to the control group, whereas arguments with 'y' as a name or
suffix correspond to the experimental group.

The equivalence interval can be specified with the argument \code{interval}.
However, it is not compulsory to specify an equivalence interval (see van
Ravenzwaaij et al., 2019). The default value of the argument \code{interval}
is 0, indicating a point null hypothesis. If an interval is preferred, the
argument \code{interval} can be set in two ways: A \emph{symmetric} interval
can be defined by either specifying a numeric vector of length one (e.g., 0.1,
which is converted to c(-0.1, 0.1)) or a numeric vector of length two (e.g.,
c(-0.1, 0.1)); an \emph{asymmetric} interval can be defined by specifying a
numeric vector of length two (e.g., c(-0.1, 0.2)). It can be specified
whether the equivalence interval (i.e., \code{interval}) is given in
standardized or unstandardized units with the \code{interval_std} argument,
where TRUE, corresponding to standardized units, is the default.

For the calculation of the Bayes factor, a Cauchy prior density centered on 0
is chosen for the effect size under the alternative hypothesis. The standard
Cauchy distribution, with a location parameter of 0 and a scale parameter of
1, resembles a standard Normal distribution, except that the Cauchy
distribution has less mass at the center but heavier tails (Liang et al.,
2008; Rouder et al., 2009). The argument \code{prior_scale} specifies the
width of the Cauchy prior, which corresponds to half of the interquartile
range. Thus, by adjusting the Cauchy prior scale with \code{prior_scale},
different ranges of expected effect sizes can be emphasized. The default
prior scale is set to r = 1 / sqrt(2).

\code{\link{equiv_bf}} creates an S4 object of class
\linkS4class{baymedrEquivalence}, which has multiple slots/entries (e.g.,
type of data, prior scale, Bayes factor, etc.; see Value). If it is desired
to store or extract solely the Bayes factor, the user can do this with
\code{\link{get_bf}}, by setting the S4 object as an argument (see Examples).
}
\examples{
## equiv_bf using raw data:

# Assign model to variable.
equiv_raw <- equiv_bf(x = rnorm(100, 10, 15),
                      y = rnorm(130, 13, 10))

# Extract Bayes factor from variable.
get_bf(equiv_raw)

# ----------
# ----------

## equiv_bf using summary statistics with data from Steiner et al. (2015).
## With a point null hypothesis:

# Assign model to variable.
equiv_sum_point <- equiv_bf(n_x = 560,
                            n_y = 538,
                            mean_x = 8.683,
                            mean_y = 8.516,
                            sd_x = 3.6,
                            sd_y = 3.6)

# Extract Bayes factor from model.
get_bf(equiv_sum_point)

# ----------
# ----------

## equiv_bf using summary statistics with data from Steiner et al. (2015).
## With an interval null hypothesis:

# Assign model to variable.
equiv_sum_interval <- equiv_bf(n_x = 560,
                               n_y = 538,
                               mean_x = 8.683,
                               mean_y = 8.516,
                               sd_x = 3.6,
                               sd_y = 3.6,
                               interval = 0.05)

# Extract Bayes factor from model.
get_bf(equiv_sum_interval)
}
\references{
Gronau, Q. F., Ly, A., & Wagenmakers, E.-J. (2020). Informed Bayesian
  t-tests. \emph{The American Statistician}, \emph{74}, 137-143.

  Liang, F., Paulo, R., Molina, G., Clyde, M. A., & Berger, J. O. (2008).
  Mixtures of g priors for Bayesian variable selection. \emph{Journal of the
  American Statistical Association}, \emph{103}, 410-423.

  Rouder, J. N., Speckman, P. L., Sun, D., Morey, R. D., & Iverson, G.
  (2009). Bayesian t tests for accepting and rejecting the null hypothesis.
  \emph{Psychonomic Bulletin & Review}, \emph{16}, 225-237.

  van Ravenzwaaij, D., Monden, R., Tendeiro, J. N., & Ioannidis, J. P. A.
  (2019). Bayes factors for superiority, non-inferiority, and equivalence
  designs. \emph{BMC Medical Research Methodology}, \emph{19}, 71.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/super_bf.R
\name{super_bf}
\alias{super_bf}
\title{Bayes factor for superiority designs}
\usage{
super_bf(
  x = NULL,
  y = NULL,
  n_x = NULL,
  n_y = NULL,
  mean_x = NULL,
  mean_y = NULL,
  sd_x = NULL,
  sd_y = NULL,
  ci_margin = NULL,
  ci_level = NULL,
  prior_scale = 1/sqrt(2),
  direction = "high"
)
}
\arguments{
\item{x}{A numeric vector of observations for the control group.}

\item{y}{A numeric vector of observations for the experimental group.}

\item{n_x}{A numeric vector of length one, specifying the sample size of the
control group.}

\item{n_y}{A numeric vector of length one, specifying the sample size of the
experimental group.}

\item{mean_x}{A numeric vector of length one, specifying the mean of the
dependent variable in the control group.}

\item{mean_y}{A numeric vector of length one, specifying the mean of the
dependent variable in the experimental group.}

\item{sd_x}{A numeric vector of length one, specifying the standard deviation
of the dependent variable in the control group. Only \code{sd_x} and
\code{sd_y} \strong{OR} \code{ci_margin} and \code{ci_level} should be
defined (see Details).}

\item{sd_y}{A numeric vector of length one, specifying the standard deviation
of the dependent variable in the experimental group. Only \code{sd_x} and
\code{sd_y} \strong{OR} \code{ci_margin} and \code{ci_level} should be
defined (see Details).}

\item{ci_margin}{A numeric vector of length one, specifying the margin of the
confidence interval (i.e., the width of the confidence interval divided by
2) of the mean difference on the dependent variable between the
experimental and control groups. The value should be a positive number Only
\code{sd_x} and \code{sd_y} \strong{OR} \code{ci_margin} and
\code{ci_level} should be defined (see Details).}

\item{ci_level}{A numeric vector of length one, specifying the confidence
level of \code{ci_margin}. The value must be between 0 and 1 (e.g., 0.95
for a 95\% confidence interval). Only \code{sd_x} and \code{sd_y}
\strong{OR} \code{ci_margin} and \code{ci_level} should be defined (see
Details).}

\item{prior_scale}{A numeric vector of length one, specifying the scale of
the Cauchy prior distribution for the effect size under the alternative
hypothesis (see Details). The default value is r = 1 / sqrt(2).}

\item{direction}{A character vector of length one, specifying the direction
of superior scores. 'low' indicates that low scores on the measure of
interest correspond to a superior outcome and 'high' (the default)
indicates that high scores on the measure of interest correspond to a
superior outcome (see Details).}
}
\value{
An S4 object of class \linkS4class{baymedrSuperiority} is returned.
  Contained are a description of the model and the resulting Bayes factor:
  \itemize{ \item test: The type of analysis \item hypotheses: A statement of
  the hypotheses \itemize{ \item h0: The null hypothesis \item h1: The
  alternative hypothesis} \item data: A description of the data \itemize{
  \item type: The type of data ('raw' when arguments \code{x} and \code{y}
  are used or 'summary' when arguments \code{n_x}, \code{n_y}, \code{mean_x},
  \code{mean_y}, \code{sd_x}, and \code{sd_y} (or \code{ci_margin} and
  \code{ci_level} instead of \code{sd_x} and \code{sd_y}) are used) \item
  ...: values for the arguments used, depending on 'raw' or 'summary'} \item
  prior_scale: The scale of the Cauchy prior distribution \item bf: The
  resulting Bayes factor } A summary of the model is shown by printing the
  object.
}
\description{
\code{\link{super_bf}} computes a Bayes factor for superiority designs with a
continuous dependent variable.
}
\details{
The formulation of the null and alternative hypotheses for the superiority
design differs depending on whether high or low scores on the dependent
variable represent superiority. In both cases, the null hypothesis (i.e., H0)
states that the population means of the experimental group and the control
group are equivalent. In the case where high scores correspond to
superiority, the alternative hypothesis states that the population mean of
the experimental group is higher than the population mean of the control
group. Thus, the alternative hypothesis goes in the positive direction (i.e.,
H+). In turn, in the case where low scores correspond to superiority, the
alternative hypothesis states that the population mean of the experimental
group is lower than the population mean of the control group. Thus, the
alternative hypothesis goes in the negative direction (i.e., H-). The
dependent variable must be continuous.

Since the main goal of \code{\link{super_bf}} is to establish superiority,
the resulting Bayes factor quantifies evidence in favor of the alternative
hypothesis. In the case where low values represent superiority we have BF-0,
whereas in the case where high values represent superiority we have BF+0.
Evidence for the null hypothesis can easily be calculated by taking the
reciprocal of the original Bayes factor (i.e., BF0- = 1 / BF-0 and BF0+ = 1 /
BF+0). Quantification of evidence in favor of the null hypothesis is
logically sound and legitimate within the Bayesian framework (see e.g., van
Ravenzwaaij et al., 2019).

\code{\link{super_bf}} can be utilized to calculate a Bayes factor based on
raw data (i.e., if arguments \code{x} and \code{y} are defined) or summary
statistics (i.e., if arguments \code{n_x}, \code{n_y}, \code{mean_x}, and
\code{mean_y} are defined). In the latter case, the user has the freedom to
supply values either for the arguments \code{sd_x} and \code{sd_y}
\strong{OR} \code{ci_margin} and \code{ci_level}. Arguments with 'x' as a
name or suffix correspond to the control group, whereas arguments with 'y' as
a name or suffix correspond to the experimental group (i.e., the group for
which we seek to establish superiority).

For the calculation of the Bayes factor, a Cauchy prior density centered on 0
is chosen for the effect size under the alternative hypothesis. The standard
Cauchy distribution, with a location parameter of 0 and a scale parameter of
1, resembles a standard Normal distribution, except that the Cauchy
distribution has less mass at the center but heavier tails (Liang et al.,
2008; Rouder et al., 2009). The argument \code{prior_scale} specifies the
width of the Cauchy prior, which corresponds to half of the interquartile
range. Thus, by adjusting the Cauchy prior scale with \code{prior_scale},
different ranges of expected effect sizes can be emphasized. The default
prior scale is set to r = 1 / sqrt(2).

\code{\link{super_bf}} creates an S4 object of class
\linkS4class{baymedrSuperiority}, which has multiple slots/entries (e.g.,
type of data, prior scale, Bayes factor, etc.; see Value). If it is desired
to store or extract solely the Bayes factor, the user can do this with
\code{\link{get_bf}}, by setting the S4 object as an argument (see Examples).
}
\examples{
## super_bf using raw data:

# Assign model to variable.
super_raw <- super_bf(x = rnorm(100, 10, 15),
                      y = rnorm(130, 13, 10))

# Extract Bayes factor from model.
get_bf(super_raw)

# ----------
# ----------

## super_bf using summary statistics with data from Skjerven et al. (2013).
## EXAMPLE 1

# Assign model to variable.
super_sum_ex1 <- super_bf(n_x = 201,
                          n_y = 203,
                          mean_x = 68.1,
                          mean_y = 63.6,
                          ci_margin = (15.5 - (-6.5)) / 2,
                          ci_level = 0.95,
                          direction = "low")

# Extract Bayes factor from model.
get_bf(super_sum_ex1)

# ----------

## super_bf using summary statistics with data from Skjerven et al. (2013).
## EXAMPLE 2

# Assign model to variable.
super_sum_ex2 <- super_bf(n_x = 200,
                          n_y = 204,
                          mean_x = 47.6,
                          mean_y = 61.3,
                          ci_margin = (24.4 - 2.9) / 2,
                          ci_level = 0.95,
                          direction = "low")

# Extract Bayes factor from model.
get_bf(super_sum_ex2)
}
\references{
Gronau, Q. F., Ly, A., & Wagenmakers, E.-J. (2020). Informed Bayesian
  t-tests. \emph{The American Statistician}, \emph{74}, 137-143.

  Liang, F., Paulo, R., Molina, G., Clyde, M. A., & Berger, J. O. (2008).
  Mixtures of g priors for Bayesian variable selection. \emph{Journal of the
  American Statistical Association}, \emph{103}, 410-423.

  Rouder, J. N., Speckman, P. L., Sun, D., Morey, R. D., & Iverson, G.
  (2009). Bayesian t tests for accepting and rejecting the null hypothesis.
  \emph{Psychonomic Bulletin & Review}, \emph{16}, 225-237.

  van Ravenzwaaij, D., Monden, R., Tendeiro, J. N., & Ioannidis, J. P. A.
  (2019). Bayes factors for superiority, non-inferiority, and equivalence
  designs. \emph{BMC Medical Research Methodology}, \emph{19}, 71.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/baymedr-package.R
\docType{package}
\name{baymedr-package}
\alias{baymedr-package}
\alias{baymedr}
\title{baymedr: Computation of Bayes Factors for Common Biomedical Designs}
\description{
\code{baymedr} provides functions for the computation of Bayes factors for
common biomedical research designs.
}
\details{
\tabular{ll}{ Package: \tab baymedr \cr Type: \tab Package \cr Version: \tab
0.2 \cr License: \tab GPL-3 \cr Date: \tab 2022-09-01 \cr}
}
\seealso{
Useful links: \itemize{ \item
  \url{https://github.com/maxlinde/baymedr} \item Report bugs at
  \url{https://github.com/maxlinde/baymedr/issues}}
}
\author{
\strong{Maintainer}: Maximilian Linde --
  \email{maximilian.linde.92@gmail.com}

  Authors: \itemize{ \item Maximilian Linde (aut, cre) \item Don van
  Ravenzwaaij (aut) \item Jorge N. Tendeiro (aut)
  \item Quentin F. Gronau (ctb) }
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_bf.R
\name{get_bf}
\alias{get_bf}
\title{Extract Bayes factor from S4 object}
\usage{
get_bf(object)
}
\arguments{
\item{object}{An S4 object of class \linkS4class{baymedrSuperiority},
\linkS4class{baymedrEquivalence}, \linkS4class{baymedrNonInferiority},
\linkS4class{baymedrCoxProportionalHazards}, or
\linkS4class{baymedrCoxProportionalHazardsMulti}.}
}
\value{
A numeric vector, providing the Bayes factor(s) from an S4 object.
}
\description{
\code{\link{get_bf}} extracts the Bayes factor from an S4 object (i.e.,
\linkS4class{baymedrSuperiority}, \linkS4class{baymedrEquivalence},
\linkS4class{baymedrNonInferiority}),
\linkS4class{baymedrCoxProportionalHazards}, and
\linkS4class{baymedrCoxProportionalHazardsMulti}.
}
\examples{
# Extract Bayes factor from a baymedrSuperiority object using raw data:
mod_super <- super_bf(x = rnorm(100, 10, 15),
                      y = rnorm(130, 13, 10))

get_bf(object = mod_super)

# Extract Bayes factor from a baymedrEquivalence object using raw data:
mod_equiv <- equiv_bf(x = rnorm(100, 10, 15),
                      y = rnorm(130, 13, 10))

get_bf(object = mod_equiv)

# Extract Bayes factor from a baymedrNonInferiority object using raw data:
mod_infer <- infer_bf(x = rnorm(100, 10, 15),
                      y = rnorm(130, 13, 10),
                      ni_margin = 1)

get_bf(object = mod_infer)

# Extract Bayes factor from a baymedrCoxProportionalHazards object:
data <- survival::aml
names(data) <- c("time", "event", "group")
data$group <- ifelse(test = data$group == "Maintained",
                     yes = 0,
                     no = 1)

mod_coxph <- coxph_bf(data = data)

get_bf(object = mod_coxph)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/coxph_data_sim.R
\name{coxph_data_sim}
\alias{coxph_data_sim}
\title{Simulate data for Cox proportional hazards regression}
\usage{
coxph_data_sim(
  n_data = 1,
  ns_c,
  ns_e,
  ne_c,
  ne_e,
  cox_hr,
  cox_hr_ci_level = 0.95,
  max_t = 100,
  cores = 1,
  ...
)
}
\arguments{
\item{n_data}{The number of datasets to be simulated. The default is 1.}

\item{ns_c}{Sample size of the control condition.}

\item{ns_e}{Sample size of the experimental condition.}

\item{ne_c}{Number of events (e.g., death) in the control condition.}

\item{ne_e}{Number of events (e.g., death) in the experimental condition.}

\item{cox_hr}{A numeric vector of length 3, indicating the hazard ratio
between the experimental and control conditions based on a Cox proportional
hazards regression model, the lower boundary of the x% confidence interval
of the hazard ratio between the experimental and control conditions based
on a Cox proportional hazards regression model, and the upper boundary of
the x% confidence interval of the hazard ratio between the experimental and
control conditions based on a Cox proportional hazards regression model,
respectively. The hazard ratio must be provided. The confidence interval
boundaries are optional; if missing they should be given as NA.}

\item{cox_hr_ci_level}{Confidence level of the x% confidence interval of the
hazard ratio between the experimental and control conditions based on a Cox
proportional hazards regression model. The default is 0.95.}

\item{max_t}{The maximum allowed survival/censoring time. The default is 100.}

\item{cores}{The number of cores to be used in the data simulation process.
The default is 1. Note that it is only useful to use more than 1 core if
more than 1 dataset is simulated; ideally, \code{n_data} should be a
multiple of \code{cores}.}

\item{...}{Arguments passed to the \code{control} argument of
\code{\link[pso]{psoptim}} (e.g. maxit, maxit.stagnate). Be aware that
\code{\link{coxph_data_sim}} uses default values that are not the
default in \code{\link[pso]{psoptim}}. Specifically,
\code{\link{coxph_data_sim}} uses \code{maxit = 5000}, and
\code{maxit.stagnate = ceiling(maxit / 5)}.}
}
\value{
A list of length \code{n_data} is returned. Each element of that list
  contains one simulated dataset and information about the optimization
  process: \itemize{ \item data: A data.frame containing the following
  columns: \itemize{ \item time: Survival/censoring times. \item event:
  Indication of whether an event happened (1) or not (0). \item group:
  Indication of whether case belongs to control condition (0) or experimental
  condition (1). } \item optim: Results of particle swarm optimization. See
  the Value section in \code{\link[pso]{psoptim}} }.
}
\description{
\code{\link{coxph_data_sim}} simulates data for Cox proportional hazards
regression models with one dichotomous independent variable based on summary
statistics.
}
\details{
Particle swarm optimization, as implemented by \code{\link[pso]{psoptim}} is
used to simulate one or multiple datasets that match certain summary
statistics. The algorithm uses as many parameters as there cases in the
dataset that is to be simulated. Therefore, using
\code{\link{coxph_data_sim}} becomes more time-consuming the larger the
sample size.

The relevant summary statistics that are used in the optimization process
are: \itemize{ \item cox_hr \itemize{ \item Hazard ratio
between the experimental and control conditions based on a Cox proportional
hazards regression model. \item Lower boundary of the x% confidence interval
of the hazard ratio between the experimental and control conditions based on
a Cox proportional hazards regression model. \item Upper boundary of the x%
confidence interval of the hazard ratio between the experimental and control
conditions based on a Cox proportional hazards regression model. } }

\code{\link{coxph_data_sim}} creates a list with as many elements as
specified by the argument \code{n_data}. Each element consists of a list that
entails the resulting simulated data and the optimization results of the data
simulation process.
}
\examples{
# Pretend we extracted the following summary statistics from an article.
ns_c <- 20
ns_e <- 56
ne_c <- 18
ne_e <- 40
cox_hr <- c(0.433, 0.242, 0.774)
cox_hr_ci_level <- 0.95

# We want to simulate 3 datasets. We do not need a very precise match of the
# summary statistics to the real summary statistics. Therefore, for
# demonstration purposes we only use 1/200 of the default number of
# optimization iterations (i.e., (1 / 200) * 5000).
sim_data <- coxph_data_sim(n_data = 3,
                           ns_c = ns_c,
                           ns_e = ns_e,
                           ne_c = ne_c,
                           ne_e = ne_e,
                           cox_hr = cox_hr,
                           cox_hr_ci_level = cox_hr_ci_level,
                           max_t = 100,
                           maxit = 25)
}
\references{
Harrell, F. R. (2015). Regression modeling strategies: Withapplications to
  linear models, logistic regression, and survival analysis (2nd ed.).
  Springer.

  Kennedy, J., & Eberhart, R. (1995). Particle swarm optimization.
  \emph{Proceedings of ICNN'95 - International Conference on Neural
  Networks}, \emph{4}, 1942-1948.

  Shi, Y., & Eberhart, R. (1998). A modified particle swarm optimizer.
  \emph{1998 IEEE International Conference on Evolutionary Computation
  Proceedings. IEEE World Congress on Computational Intelligence}, 69-73.
}
\seealso{
\code{\link{coxph_bf}} and \code{\link[pso]{psoptim}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/infer_bf.R
\name{infer_bf}
\alias{infer_bf}
\title{Bayes factor for non-inferiority designs}
\usage{
infer_bf(
  x = NULL,
  y = NULL,
  n_x = NULL,
  n_y = NULL,
  mean_x = NULL,
  mean_y = NULL,
  sd_x = NULL,
  sd_y = NULL,
  ci_margin = NULL,
  ci_level = NULL,
  ni_margin = NULL,
  ni_margin_std = TRUE,
  prior_scale = 1/sqrt(2),
  direction = "high"
)
}
\arguments{
\item{x}{A numeric vector of observations for the control group.}

\item{y}{A numeric vector of observations for the experimental group.}

\item{n_x}{A numeric vector of length one, specifying the sample size of the
control group.}

\item{n_y}{A numeric vector of length one, specifying the sample size of the
experimental group.}

\item{mean_x}{A numeric vector of length one, specifying the mean of the
dependent variable in the control group.}

\item{mean_y}{A numeric vector of length one, specifying the mean of the
dependent variable in the experimental group.}

\item{sd_x}{A numeric vector of length one, specifying the standard deviation
of the dependent variable in the control group. Only \code{sd_x} and
\code{sd_y} \strong{OR} \code{ci_margin} and \code{ci_level} should be
defined (see Details).}

\item{sd_y}{A numeric vector of length one, specifying the standard deviation
of the dependent variable in the experimental group. Only \code{sd_x} and
\code{sd_y} \strong{OR} \code{ci_margin} and \code{ci_level} should be
defined (see Details).}

\item{ci_margin}{A numeric vector of length one, specifying the margin of the
confidence interval (i.e., the width of the confidence interval divided by
2) of the mean difference on the dependent variable between the
experimental and control groups. The value should be a positive number Only
\code{sd_x} and \code{sd_y} \strong{OR} \code{ci_margin} and
\code{ci_level} should be defined (see Details).}

\item{ci_level}{A numeric vector of length one, specifying the confidence
level of \code{ci_margin}. The value must be between 0 and 1 (e.g., 0.95
for a 95\% confidence interval). Only \code{sd_x} and \code{sd_y}
\strong{OR} \code{ci_margin} and \code{ci_level} should be defined (see
Details).}

\item{ni_margin}{A numeric vector of length one, specifying the
non-inferiority margin. The value should be a positive number.}

\item{ni_margin_std}{A logical vector of length one, specifying whether the
non-inferiority margin (i.e., \code{ni_margin}) is given in standardized
(TRUE; the default) or unstandardized (FALSE) units.}

\item{prior_scale}{A numeric vector of length one, specifying the scale of
the Cauchy prior distribution for the effect size under the alternative
hypothesis (see Details). The default value is r = 1 / sqrt(2).}

\item{direction}{A character vector of length one, specifying the direction
of non-inferior scores. 'low' indicates that low scores on the measure of
interest correspond to a non-inferior outcome and 'high' (the default)
indicates that high scores on the measure of interest correspond to a
non-inferior outcome (see Details).}
}
\value{
An S4 object of class \linkS4class{baymedrNonInferiority} is
  returned. Contained are a description of the model and the resulting Bayes
  factor: \itemize{ \item test: The type of analysis \item hypotheses: A
  statement of the hypotheses \itemize{ \item h0: The null hypothesis \item
  h1: The alternative hypothesis} \item ni_margin: The value for ni_margin in
  standardized and unstandardized units \itemize{ \item ni_mar_std: The
  standardized non-inferiority margin \item ni_mar_unstd: The unstandardized
  non-inferiority margin} \item data: A description of the data \itemize{
  \item type: The type of data ('raw' when arguments \code{x} and \code{y}
  are used or 'summary' when arguments \code{n_x}, \code{n_y}, \code{mean_x},
  \code{mean_y}, \code{sd_x}, and \code{sd_y} (or \code{ci_margin} and
  \code{ci_level} instead of \code{sd_x} and \code{sd_y}) are used) \item
  ...: values for the arguments used, depending on 'raw' or summary'} \item
  prior_scale: The width of the Cauchy prior distribution \item bf: The
  resulting Bayes factor } A summary of the model is shown by printing the
  object.
}
\description{
\code{\link{infer_bf}} computes a Bayes factor for non-inferiority designs
with a continuous dependent variable.
}
\details{
The formulation of the null and alternative hypotheses for the
non-inferiority design differs depending on whether high or low scores on the
dependent variable represent non-inferiority. In the case where high scores
correspond to non-inferiority, the hypotheses are as follows: The null
hypothesis states that the population mean of the experimental group (e.g., a
new medication) is lower than the population mean of the control group (e.g.,
a placebo or an already existing medication) minus the non-inferiority
margin. The alternative hypothesis states that the population mean of the
experimental group is higher than the population mean of the control group
minus the non-inferiority margin. Thus, the null hypothesis goes in the
negative direction (i.e., H-) and the alternative hypothesis in the positive
direction (i.e., H+). In turn, in the case where low scores correspond to
non-inferiority, the hypotheses are as follows: The null hypothesis states
that the population mean of the experimental group is higher than the
population mean of the control group plus the non-inferiority margin. The
alternative hypothesis states that the population mean of the experimental
group is lower than the population mean of the control group plus the
non-inferiority margin. Thus, the null hypothesis goes in the positive
direction (i.e., H+) and the alternative hypothesis in the negative direction
(i.e., H-). The dependent variable must be continuous.

Since the main goal of \code{\link{infer_bf}} is to establish
non-inferiority, the resulting Bayes factor quantifies evidence in favor of
the alternative hypothesis. In the case where high values represent
non-inferiority we have BF+- and in the case where low values represent
non-inferiority we have BF-+. Evidence for the null hypothesis can easily be
calculated by taking the reciprocal of the original Bayes factor (i.e., BF+-
= 1 / BF-+ and vice versa). Quantification of evidence in favor of the null
hypothesis is logically sound and legitimate within the Bayesian framework
(see e.g., van Ravenzwaaij et al., 2019).

\code{\link{infer_bf}} can be utilized to calculate a Bayes factor based on
raw data (i.e., if arguments \code{x} and \code{y} are defined) or summary
statistics (i.e., if arguments \code{n_x}, \code{n_y}, \code{mean_x}, and
\code{mean_y} (or \code{ci_margin} and \code{ci_level} instead of \code{sd_x}
and \code{sd_y}) are defined). Arguments with 'x' as a name or suffix
correspond to the control group, whereas arguments with 'y' as a name or
suffix correspond to the experimental group.

Since sometimes high scores on the dependent variable are considered
non-inferior (e.g., amount of social interactions) and sometimes rather the
low scores (e.g., severity of symptoms), the direction of non-inferiority can
be specified with the argument \code{direction}. For the case where high
values on the dependent variable indicate non-inferiority, 'high' (the
default) should be specified for the argument \code{direction}; if low values
on the dependent variable indicate non-inferiority, 'low' should be specified
for the argument \code{direction}.

With the argument \code{ni_margin}, the non-inferiority margin can be
specified. \code{ni_margin} should be a positive number.' It can be declared
whether the non-inferiority margin is specified in standardized or
unstandardized units with the \code{ni_margin_std} argument, where TRUE,
corresponding to standardized units, is the default.

For the calculation of the Bayes factor, a Cauchy prior density centered on 0
is chosen for the effect size under the alternative hypothesis. The standard
Cauchy distribution, with a location parameter of 0 and a scale parameter of
1, resembles a standard Normal distribution, except that the Cauchy
distribution has less mass at the center but heavier tails (Liang et al.,
2008; Rouder et al., 2009). The argument \code{prior_scale} specifies the
width of the Cauchy prior, which corresponds to half of the interquartile
range. Thus, by adjusting the Cauchy prior scale with \code{prior_scale},
different ranges of expected effect sizes can be emphasized. The default
prior scale is set to r = 1 / sqrt(2).

\code{\link{infer_bf}} creates an S4 object of class
\linkS4class{baymedrNonInferiority}, which has multiple slots/entries (e.g.,
type of data, prior scale, Bayes factor, etc.; see Value). If it is desired
to store or extract solely the Bayes factor, the user can do this with
\code{\link{get_bf}}, by setting the S4 object as an argument (see Examples).
}
\examples{
## infer_bf using raw data:

# Assign model to variable.
infer_raw <- infer_bf(x = rnorm(100, 10, 15),
                      y = rnorm(130, 13, 10),
                      ni_margin = 1.5,
                      ni_margin_std = FALSE)

# Extract Bayes factor from model.
get_bf(infer_raw)

# ----------
# ----------

## infer_bf using summary statistics with data from Andersson et al. (2013).
## Test at timepoint 1:

# Assign model to variable.
infer_sum_t1 <- infer_bf(n_x = 33,
                         n_y = 32,
                         mean_x = 17.1,
                         mean_y = 13.6,
                         sd_x = 8,
                         sd_y = 9.8,
                         ni_margin = 2,
                         ni_margin_std = FALSE,
                         direction = "low")

# Extract Bayes factor from model
get_bf(infer_sum_t1)

# ----------
# ----------

## infer_bf using summary statistics with data from Andersson et al. (2013).
## Test at timepoint 2:

# Assign model to variable.
infer_sum_t2 <- infer_bf(n_x = 30,
                         n_y = 32,
                         mean_x = 13.5,
                         mean_y = 9.2,
                         sd_x = 8.7,
                         sd_y = 7.6,
                         ni_margin = 2,
                         ni_margin_std = FALSE,
                         direction = "low")

# Extract Bayes factor from model
get_bf(infer_sum_t2)
}
\references{
Gronau, Q. F., Ly, A., & Wagenmakers, E.-J. (2020). Informed Bayesian
  t-tests. \emph{The American Statistician}, \emph{74}, 137-143.

  Liang, F., Paulo, R., Molina, G., Clyde, M. A., & Berger, J. O. (2008).
  Mixtures of g priors for Bayesian variable selection. \emph{Journal of the
  American Statistical Association}, \emph{103}, 410-423.

  Rouder, J. N., Speckman, P. L., Sun, D., Morey, R. D., & Iverson, G.
  (2009). Bayesian t tests for accepting and rejecting the null hypothesis.
  \emph{Psychonomic Bulletin & Review}, \emph{16}, 225-237.

  van Ravenzwaaij, D., Monden, R., Tendeiro, J. N., & Ioannidis, J. P. A.
  (2019). Bayes factors for superiority, non-inferiority, and equivalence
  designs. \emph{BMC Medical Research Methodology}, \emph{19}, 71.
}
